import glob
import os
from multiprocessing import Pool

from apex.core.calculator.calculator import make_calculator
from apex.core.property.Elastic import Elastic
from apex.core.property.EOS import EOS
from apex.core.property.Gamma import Gamma
from apex.core.property.Interstitial import Interstitial
from apex.core.lib.utils import create_path
from apex.core.lib.util import collect_task
from apex.core.lib.dispatcher import make_submission
from apex.core.property.Surface import Surface
from apex.core.property.Vacancy import Vacancy
from apex.core.property.Phonon import Phonon
from apex.utils import sepline, get_task_type
from dflow.python import upload_packages
upload_packages.append(__file__)

lammps_task_type = ["deepmd", "meam", "eam_fs", "eam_alloy"]


def make_property_instance(parameters, inter_param):
    """
    Make an instance of Property
    """
    prop_type = parameters["type"]
    if prop_type == "eos":
        return EOS(parameters, inter_param)
    elif prop_type == "elastic":
        return Elastic(parameters, inter_param)
    elif prop_type == "vacancy":
        return Vacancy(parameters, inter_param)
    elif prop_type == "interstitial":
        return Interstitial(parameters, inter_param)
    elif prop_type == "surface":
        return Surface(parameters, inter_param)
    elif prop_type == "gamma":
        return Gamma(parameters, inter_param)
    elif prop_type == "phonon":
        return Phonon(parameters, inter_param)
    else:
        raise RuntimeError(f"unknown dflowautotest type {prop_type}")


def make_property(confs, inter_param, property_list):
    # find all POSCARs and their name like mp-xxx
    # ...
    # conf_dirs = glob.glob(confs)
    # conf_dirs.sort()
    conf_dirs = []
    for conf in confs:
        conf_dirs.extend(glob.glob(conf))
    conf_dirs.sort()
    for ii in conf_dirs:
        sepline(ch=ii, screen=True)
        for jj in property_list:
            if jj.get("skip", False):
                continue
            if "init_from_suffix" and "output_suffix" in jj:
                do_refine = True
                suffix = jj["output_suffix"]
            elif "reproduce" in jj and jj["reproduce"]:
                do_refine = False
                suffix = "reprod"
            elif 'suffix' in jj and jj['suffix']:
                do_refine = False
                suffix = str(jj['suffix'])
            else:
                do_refine = False
                suffix = "00"
            # generate working directory like mp-xxx/eos_00 if jj['type'] == 'eos'
            # handel the exception that the working directory exists
            # ...

            # determine the suffix: from scratch or refine
            # ...

            property_type = jj["type"]
            path_to_equi = os.path.join(ii, "relaxation", "relax_task")
            path_to_work = os.path.join(ii, property_type + "_" + suffix)

            create_path(path_to_work)

            inter_param_prop = inter_param
            if "cal_setting" in jj and "overwrite_interaction" in jj["cal_setting"]:
                inter_param_prop = jj["cal_setting"]["overwrite_interaction"]

            prop = make_property_instance(jj, inter_param_prop)
            task_list = prop.make_confs(path_to_work, path_to_equi, do_refine)

            for kk in task_list:
                poscar = os.path.join(kk, "POSCAR")
                inter = make_calculator(inter_param_prop, poscar)
                inter.make_potential_files(kk)
                #dlog.debug(prop.task_type())  ### debug
                inter.make_input_file(kk, prop.task_type(), prop.task_param())

            prop.post_process(
                task_list
            )  # generate same KPOINTS file for elastic when doing VASP


def worker(
    work_path,
    all_task,
    forward_common_files,
    forward_files,
    backward_files,
    mdata,
    inter_type,
    task_type,
):
    run_tasks = [os.path.basename(ii) for ii in all_task]
    machine = mdata.get("machine", None)
    resources = mdata.get("resources", None)
    command = mdata.get(f"{task_type}_run_command", None)
    if not command:
        command = mdata.get("run_command", None)
    group_size = mdata.get("group_size", 1)
    submission = make_submission(
        mdata_machine=machine,
        mdata_resources=resources,
        commands=[command],
        work_path=work_path,
        run_tasks=run_tasks,
        group_size=group_size,
        forward_common_files=forward_common_files,
        forward_files=forward_files,
        backward_files=backward_files,
        outlog="outlog",
        errlog="errlog",
    )
    submission.run_submission()


def run_property(confs, inter_param, property_list, mdata):
    # find all POSCARs and their name like mp-xxx
    # ...
    # conf_dirs = glob.glob(confs)
    # conf_dirs.sort()
    processes = len(property_list)
    pool = Pool(processes=processes)
    print("Submit job via %d processes" % processes)
    conf_dirs = []
    for conf in confs:
        conf_dirs.extend(glob.glob(conf))
    conf_dirs.sort()
    task_list = []
    work_path_list = []
    multiple_ret = []
    for ii in conf_dirs:
        sepline(ch=ii, screen=True)
        for jj in property_list:
            # determine the suffix: from scratch or refine
            # ...
            if jj.get("skip", False):
                continue
            if "init_from_suffix" and "output_suffix" in jj:
                suffix = jj["output_suffix"]
            elif "reproduce" in jj and jj["reproduce"]:
                suffix = "reprod"
            elif 'suffix' in jj and jj['suffix']:
                suffix = str(jj['suffix'])
            else:
                suffix = "00"

            property_type = jj["type"]
            path_to_work = os.path.abspath(
                os.path.join(ii, property_type + "_" + suffix)
            )

            work_path_list.append(path_to_work)
            tmp_task_list = glob.glob(os.path.join(path_to_work, "task.[0-9]*[0-9]"))
            tmp_task_list.sort()
            task_list.append(tmp_task_list)

            inter_param_prop = inter_param
            if "cal_setting" in jj and "overwrite_interaction" in jj["cal_setting"]:
                inter_param_prop = jj["cal_setting"]["overwrite_interaction"]

            # dispatch the tasks
            # POSCAR here is useless
            virtual_calculator = make_calculator(inter_param_prop, "POSCAR")
            forward_files = virtual_calculator.forward_files(property_type)
            forward_common_files = virtual_calculator.forward_common_files(
                property_type
            )
            backward_files = virtual_calculator.backward_files(property_type)
            #    backward_files += logs
            # ...
            task_type = get_task_type({"interaction": inter_param})
            inter_type = inter_param_prop["type"]
            work_path = path_to_work
            all_task = tmp_task_list
            run_tasks = collect_task(all_task, inter_type)
            if len(run_tasks) == 0:
                continue
            else:
                ret = pool.apply_async(
                    worker,
                    (
                        work_path,
                        all_task,
                        forward_common_files,
                        forward_files,
                        backward_files,
                        mdata,
                        inter_type,
                        task_type
                    ),
                )
                multiple_ret.append(ret)
    pool.close()
    pool.join()
    for ii in range(len(multiple_ret)):
        if not multiple_ret[ii].successful():
            print("ERROR:", multiple_ret[ii].get())
            raise RuntimeError("Job %d is not successful!" % ii)
    print("%d jobs are finished" % len(multiple_ret))


def post_property(confs, inter_param, property_list):
    # find all POSCARs and their name like mp-xxx
    # ...
    #    task_list = []
    # conf_dirs = glob.glob(confs)
    # conf_dirs.sort()
    conf_dirs = []
    for conf in confs:
        conf_dirs.extend(glob.glob(conf))
    conf_dirs.sort()
    for ii in conf_dirs:
        for jj in property_list:
            # determine the suffix: from scratch or refine
            # ...
            if jj.get("skip", False):
                continue
            if "init_from_suffix" and "output_suffix" in jj:
                suffix = jj["output_suffix"]
            elif "reproduce" in jj and jj["reproduce"]:
                suffix = "reprod"
            elif 'suffix' in jj and jj['suffix']:
                suffix = str(jj['suffix'])
            else:
                suffix = "00"

            inter_param_prop = inter_param
            if "cal_setting" in jj and "overwrite_interaction" in jj["cal_setting"]:
                inter_param_prop = jj["cal_setting"]["overwrite_interaction"]

            property_type = jj["type"]
            path_to_work = os.path.join(ii, property_type + "_" + suffix)
            prop = make_property_instance(jj, inter_param_prop)
            prop.compute(
                os.path.join(path_to_work, "result.json"),
                os.path.join(path_to_work, "result.out"),
                path_to_work,
            )



import glob
import json
import os
import shutil
import sys
import unittest

import dpdata
import numpy as np
from monty.serialization import dumpfn, loadfn

from pymatgen.io.vasp import Incar

from apex.core.calculator.calculator import make_calculator
from apex.core.common_equi import make_equi, post_equi

#from .context import make_kspacing_kpoints, setUpModule

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestEqui(unittest.TestCase):
    jdata = {
        "structures": ["confs/std-fcc"],
        "interaction": {
            "type": "vasp",
            "incar": "vasp_input/INCAR.rlx",
            "potcar_prefix": ".",
            "potcars": {"Al": "vasp_input/POT_Al"},
        },
        "relaxation": {
            "cal_type": "relaxation",
            "cal_setting": {"input_prop": "vasp_input/INCAR"},
        },
    }

    def tearDown(self):
        if os.path.isfile("confs/std-fcc/POSCAR"):
            os.remove("confs/std-fcc/POSCAR")
        if os.path.exists("confs/std-fcc/relaxation"):
            shutil.rmtree("confs/std-fcc/relaxation")

    def test_make_equi(self):
        confs = self.jdata["structures"]
        inter_param = self.jdata["interaction"]
        relax_param = self.jdata["relaxation"]
        make_equi(confs, inter_param, relax_param)

        self.assertTrue(os.path.isfile("confs/std-fcc/POSCAR"))

        target_path = "confs/std-fcc/relaxation/relax_task"
        source_path = "vasp_input"

        incar0 = Incar.from_file(os.path.join("vasp_input", "INCAR"))
        incar1 = Incar.from_file(os.path.join(target_path, "INCAR"))
        self.assertTrue(incar0 == incar1)

        with open(os.path.join("vasp_input", "POT_Al")) as fp:
            pot0 = fp.read()
        with open(os.path.join(target_path, "POTCAR")) as fp:
            pot1 = fp.read()
        self.assertEqual(pot0, pot1)

        self.assertTrue(os.path.isfile(os.path.join(target_path, "KPOINTS")))

        task_json_file = os.path.join(target_path, "task.json")
        self.assertTrue(os.path.isfile(task_json_file))
        task_json = loadfn(task_json_file)
        self.assertEqual(task_json, relax_param)

        inter_json_file = os.path.join(target_path, "inter.json")
        self.assertTrue(os.path.isfile(inter_json_file))
        inter_json = loadfn(inter_json_file)
        self.assertEqual(inter_json, inter_param)

        self.assertTrue(os.path.islink(os.path.join(target_path, "POSCAR")))



import os

from pymatgen.ext.matproj import MPRester, MPRestError
from dflow.python import upload_packages
upload_packages.append(__file__)


web = "materials.org"


def check_apikey():
    try:
        apikey = os.environ["MAPI_KEY"]
    except KeyError:
        print("You have to get a MAPI_KEY from " + web)
        print("and execute following command:")
        print('echo "export MAPI_KEY=yourkey">> ~/.bashrc')
        print("source ~/.bashrc")
        os._exit(0)
    try:
        return MPRester(apikey)
    except MPRestError:
        #dlog.info("MPRester Error, you need to prepare POSCAR manually")
        print("MPRester Error, you need to prepare POSCAR manually")
        os._exit(0)


def get_structure(mp_id):
    mpr = check_apikey()
    return mpr.get_structure_by_material_id(mp_id)



import glob
import os
import shutil
import sys
import unittest

import dpdata
import numpy as np
from monty.serialization import loadfn
from pymatgen.io.vasp import Incar
from apex.core.property.EOS import EOS

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestEOS(unittest.TestCase):
    def setUp(self):
        _jdata = {
            "structures": ["confs/std-fcc"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR.rlx",
                "potcar_prefix": ".",
                "potcars": {"Li": "vasp_input/POTCAR"},
            },
            "properties": [
                {
                    "type": "eos",
                    "skip": False,
                    "vol_start": 0.8,
                    "vol_end": 1.2,
                    "vol_step": 0.01,
                    "cal_setting": {
                        "relax_pos": True,
                        "relax_shape": True,
                        "relax_vol": False,
                        "overwrite_interaction": {
                            "type": "deepmd",
                            "model": "lammps_input/frozen_model.pb",
                            "type_map": {"Al": 0},
                        },
                    },
                }
            ],
        }

        self.equi_path = "confs/std-fcc/relaxation/relax_task"
        self.source_path = "equi/vasp"
        self.target_path = "confs/std-fcc/eos_00"
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata["properties"]

        self.eos = EOS(_jdata["properties"][0])

    def tearDown(self):
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

    def test_task_type(self):
        self.assertEqual("eos", self.eos.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.eos.task_param())

    def test_make_confs_0(self):

        if not os.path.exists(os.path.join(self.equi_path, "CONTCAR")):
            with self.assertRaises(RuntimeError):
                self.eos.make_confs(self.target_path, self.equi_path)
        shutil.copy(
            os.path.join(self.source_path, "CONTCAR"),
            os.path.join(self.equi_path, "CONTCAR"),
        )
        task_list = self.eos.make_confs(self.target_path, self.equi_path)
        dfm_dirs = glob.glob(os.path.join(self.target_path, "task.*"))

        incar0 = Incar.from_file(os.path.join("vasp_input", "INCAR.rlx"))
        incar0["ISIF"] = 4

        for ii in dfm_dirs:
            self.assertTrue(os.path.isfile(os.path.join(ii, "POSCAR")))
            eos_json_file = os.path.join(ii, "eos.json")
            self.assertTrue(os.path.isfile(eos_json_file))
            eos_json = loadfn(eos_json_file)
            self.assertEqual(
                os.path.realpath(os.path.join(ii, "POSCAR.orig")),
                os.path.realpath(os.path.join(self.equi_path, "CONTCAR")),
            )
            sys = dpdata.System(os.path.join(ii, "POSCAR"))
            natoms = sys.get_natoms()
            self.assertAlmostEqual(
                eos_json["volume"], np.linalg.det(sys["cells"][0]) / natoms
            )

    def test_make_confs_1(self):
        self.eos.reprod = True
        shutil.copy(
            os.path.join(self.source_path, "CONTCAR"),
            os.path.join(self.equi_path, "CONTCAR"),
        )
        with self.assertRaises(RuntimeError):
            self.eos.make_confs(self.target_path, self.equi_path)



import os
import re

import numpy as np
import requests

#from dpgen import dlog
from apex.core.calculator.lib import abacus_utils, lammps_utils, vasp_utils
from apex.core.lib.utils import cmd_append_log
from dflow.python import upload_packages
upload_packages.append(__file__)

lammps_task_type = ["deepmd", "meam", "eam_fs", "eam_alloy"]  # 06/13 revised


def voigt_to_stress(inpt):
    ret = np.zeros((3, 3))
    ret[0][0] = inpt[0]
    ret[1][1] = inpt[1]
    ret[2][2] = inpt[2]
    ret[0][1] = inpt[3]
    ret[0][2] = inpt[4]
    ret[1][2] = inpt[5]
    ret[2][0] = ret[0][2]
    ret[1][0] = ret[0][1]
    ret[2][1] = ret[1][2]
    return ret


def insert_data(task, task_type, username, file_name):
    assert task in ["eos", "elastic", "surf"]
    assert task_type in ["vasp", "deepmd"]
    url = (
        "http://115.27.161.2:5000/insert_test_data?username=%s&expr_type=%s&data_type=%s"
        % (username, task_type, task)
    )
    res = requests.post(url, data=open(file_name).read())
    print("Successful upload!")


def make_work_path(jdata, task, reprod_opt, static, user):

    task_type = jdata["task_type"]
    conf_dir = jdata["conf_dir"]
    conf_path = os.path.abspath(conf_dir)
    task_path = re.sub("confs", task, conf_path)

    if task_type == "vasp":
        if user:
            work_path = os.path.join(task_path, "vasp-user_incar")
            assert os.path.isdir(work_path)
            return work_path
        if static:
            if "scf_incar" in jdata.keys():
                task_type = task_type + "-static-scf_incar"
            else:
                kspacing = jdata["vasp_params"]["kspacing"]
                task_type = task_type + "-static-k%.2f" % (kspacing)
        else:
            if "relax_incar" in jdata.keys():
                task_type = task_type + "-relax_incar"
            else:
                kspacing = jdata["vasp_params"]["kspacing"]
                task_type = task_type + "-k%.2f" % (kspacing)
    elif task_type in lammps_task_type:
        if static:
            task_type = task_type + "-static"
        elif reprod_opt:
            if "relax_incar" in jdata.keys():
                task_type = task_type + "-reprod-relax_incar"
            else:
                kspacing = jdata["vasp_params"]["kspacing"]
                task_type = task_type + "-reprod-k%.2f" % (kspacing)

    work_path = os.path.join(task_path, task_type)
    assert os.path.isdir(work_path)
    return work_path


def get_machine_info(mdata, task_type):
    if task_type in ["vasp", "abacus"]:
        vasp_exec = mdata["fp_command"]
        group_size = mdata["fp_group_size"]
        resources = mdata["fp_resources"]
        machine = mdata["fp_machine"]
        command = vasp_exec
        command = cmd_append_log(command, "log")
    elif task_type in lammps_task_type:
        model_devi_exec = mdata["model_devi_command"]
        group_size = mdata["model_devi_group_size"]
        resources = mdata["model_devi_resources"]
        machine = mdata["model_devi_machine"]
        command = model_devi_exec + " -i in.lammps"
        command = cmd_append_log(command, "model_devi.log")
    return machine, resources, command, group_size


def collect_task(all_task, task_type):

    if task_type == "vasp":
        output_file = "OUTCAR"
        check_finished = vasp_utils.check_finished
    elif task_type in lammps_task_type:
        output_file = "log.lammps"
        check_finished = lammps_task_type.check_finished
    elif task_type == "abacus":
        output_file = "OUT.ABACUS/running_relax.log"
        check_finished = abacus_utils.check_finished

    run_tasks_ = []
    for ii in all_task:
        fres = os.path.join(ii, output_file)
        if os.path.isfile(fres):
            if not check_finished(fres):
                run_tasks_.append(ii)
        else:
            run_tasks_.append(ii)

    run_tasks = [os.path.basename(ii) for ii in run_tasks_]
    return run_tasks



import os

from dflow.python import upload_packages
upload_packages.append(__file__)


def recursive_search(directories, path='.'):
    # list all directions
    items = os.listdir(path)
    directories_in_path = [
        i for i in items if os.path.isdir(os.path.join(path, i)) and not i.startswith('.')
    ]

    # check if target work direction is found
    if set(directories) <= set(directories_in_path):
        return os.path.abspath(path)

    # recursive search in next direction
    if len(directories_in_path) == 1:
        return recursive_search(directories, os.path.join(path, directories_in_path[0]))

    # return False for failure
    return False



import os
import warnings
import logging

from monty.serialization import dumpfn, loadfn

from apex.core.calculator.lib import lammps_utils
#from dpgen import dlog
from apex.core.calculator.lib.lammps_utils import (
    inter_deepmd,
    inter_eam_alloy,
    inter_eam_fs,
    inter_meam,
)
from .Task import Task
from dflow.python import upload_packages
upload_packages.append(__file__)

supported_inter = ["deepmd", "meam", "eam_fs", "eam_alloy"]


class Lammps(Task):
    def __init__(self, inter_parameter, path_to_poscar):
        self.inter = inter_parameter
        self.inter_type = inter_parameter["type"]
        self.type_map = inter_parameter["type_map"]
        self.in_lammps = inter_parameter.get("in_lammps", "auto")
        if self.inter_type == "meam":
            self.model = list(map(os.path.abspath, inter_parameter["model"]))
        else:
            self.model = os.path.abspath(inter_parameter["model"])
        self.path_to_poscar = path_to_poscar
        assert self.inter_type in supported_inter
        self.set_inter_type_func()

    def set_inter_type_func(self):

        if self.inter_type == "deepmd":
            self.inter_func = inter_deepmd

        elif self.inter_type == "meam":
            self.inter_func = inter_meam

        elif self.inter_type == "eam_fs":
            self.inter_func = inter_eam_fs

        else:
            self.inter_func = inter_eam_alloy

    def set_model_param(self):

        if self.inter_type == "deepmd":
            model_name = os.path.basename(self.model)
            deepmd_version = self.inter.get("deepmd_version", "1.2.0")
            self.model_param = {
                "model_name": [model_name],
                "param_type": self.type_map,
                "deepmd_version": deepmd_version,
            }
        elif self.inter_type == "meam":
            model_name = list(map(os.path.basename, self.model))
            self.model_param = {"model_name": model_name, "param_type": self.type_map}
        else:
            model_name = os.path.basename(self.model)
            self.model_param = {"model_name": [model_name], "param_type": self.type_map}

    def make_potential_files(self, output_dir):
        cwd = os.getcwd()
        if self.inter_type == "meam":
            model_lib = os.path.basename(self.model[0])
            model_file = os.path.basename(self.model[1])
            os.chdir(os.path.join(output_dir, "../../"))
            if os.path.islink(model_lib):
                link_lib = os.readlink(model_lib)
                if not os.path.abspath(link_lib) == self.model[0]:
                    os.remove(model_lib)
                    os.symlink(os.path.relpath(self.model[0]), model_lib)
            elif os.path.isfile(model_lib):
                os.remove(model_lib)
                os.symlink(os.path.relpath(self.model[0]), model_lib)
            else:
                os.symlink(os.path.relpath(self.model[0]), model_lib)

            if os.path.islink(model_file):
                link_file = os.readlink(model_file)
                if not os.path.abspath(link_file) == self.model[1]:
                    os.remove(model_file)
                    os.symlink(os.path.relpath(self.model[1]), model_file)
            elif os.path.isfile(model_lib):
                os.remove(model_lib)
                os.symlink(os.path.relpath(self.model[1]), model_file)
            else:
                os.symlink(os.path.relpath(self.model[1]), model_file)
            os.chdir(output_dir)
            if not os.path.islink(model_lib):
                os.symlink(os.path.join("../..", model_lib), model_lib)
            elif not os.path.join("../..", model_lib) == os.readlink(model_lib):
                os.remove(model_lib)
                os.symlink(os.path.join("../..", model_lib), model_lib)

            if not os.path.islink(model_file):
                os.symlink(os.path.join("../..", model_file), model_file)
            elif not os.path.join("../..", model_file) == os.readlink(model_file):
                os.remove(model_file)
                os.symlink(os.path.join("../..", model_file), model_file)
            os.chdir(cwd)

        else:
            model_file = os.path.basename(self.model)
            os.chdir(os.path.join(output_dir, "../../"))
            if os.path.islink(model_file):
                link_file = os.readlink(model_file)
                if not os.path.abspath(link_file) == self.model:
                    os.remove(model_file)
                    os.symlink(os.path.relpath(self.model), model_file)
            elif os.path.isfile(model_file):
                os.remove(model_file)
                os.symlink(os.path.relpath(self.model), model_file)
            else:
                os.symlink(os.path.relpath(self.model), model_file)
            os.chdir(output_dir)
            if not os.path.islink(model_file):
                os.symlink(os.path.join("../..", model_file), model_file)
            elif not os.path.join("../..", model_file) == os.readlink(model_file):
                os.remove(model_file)
                os.symlink(os.path.join("../..", model_file), model_file)
            os.chdir(cwd)

        dumpfn(self.inter, os.path.join(output_dir, "inter.json"), indent=4)

    def make_input_file(self, output_dir, task_type, task_param):
        lammps_utils.cvt_lammps_conf(
            os.path.join(output_dir, "POSCAR"),
            os.path.join(output_dir, "conf.lmp"),
            lammps_utils.element_list(self.type_map),
        )

        # dumpfn(task_param, os.path.join(output_dir, 'task.json'), indent=4)

        etol = 0
        ftol = 1e-10
        maxiter = 5000
        maxeval = 500000
        B0 = 70
        bp = 0
        ntypes = len(self.type_map)

        cal_type = task_param["cal_type"]
        cal_setting = task_param["cal_setting"]

        self.set_model_param()

        # deal with user input in.lammps for relaxation
        if os.path.isfile(self.in_lammps) and task_type == "relaxation":
            with open(self.in_lammps, "r") as fin:
                fc = fin.read()
        # user input in.lammps for apex calculation
        elif "input_prop" in cal_setting \
                and os.path.isfile(cal_setting["input_prop"]) \
                and not task_type == "relaxation":
            with open(os.path.abspath(cal_setting["input_prop"]), "r") as fin:
                fc = fin.read()

        else:
            logging.basicConfig(level=logging.INFO, filename='info.log', filemode='w',
                                format='%(asctime)s - %(levelname)s - %(message)s')
            if "etol" in cal_setting:
                logging.info(
                    "%s setting etol to %s"
                    % (self.make_input_file.__name__, cal_setting["etol"])
                )
                etol = cal_setting["etol"]
            if "ftol" in cal_setting:
                logging.info(
                    "%s setting ftol to %s"
                    % (self.make_input_file.__name__, cal_setting["ftol"])
                )
                ftol = cal_setting["ftol"]
            if "maxiter" in cal_setting:
                logging.info(
                    "%s setting maxiter to %s"
                    % (self.make_input_file.__name__, cal_setting["maxiter"])
                )
                maxiter = cal_setting["maxiter"]
            if "maxeval" in cal_setting:
                logging.info(
                    "%s setting maxeval to %s"
                    % (self.make_input_file.__name__, cal_setting["maxeval"])
                )
                maxeval = cal_setting["maxeval"]

            if cal_type == "relaxation":
                relax_pos = cal_setting["relax_pos"]
                relax_shape = cal_setting["relax_shape"]
                relax_vol = cal_setting["relax_vol"]

                if [relax_pos, relax_shape, relax_vol] == [True, False, False]:
                    fc = lammps_utils.make_lammps_equi(
                        "conf.lmp",
                        self.type_map,
                        self.inter_func,
                        self.model_param,
                        etol,
                        ftol,
                        maxiter,
                        maxeval,
                        False,
                    )
                elif [relax_pos, relax_shape, relax_vol] == [True, True, True]:
                    fc = lammps_utils.make_lammps_equi(
                        "conf.lmp",
                        self.type_map,
                        self.inter_func,
                        self.model_param,
                        etol,
                        ftol,
                        maxiter,
                        maxeval,
                        True,
                    )
                elif [relax_pos, relax_shape, relax_vol] == [
                    True,
                    True,
                    False,
                ] and not task_type == "eos":
                    if "scale2equi" in task_param:
                        scale2equi = task_param["scale2equi"]
                        fc = lammps_utils.make_lammps_press_relax(
                            "conf.lmp",
                            self.type_map,
                            scale2equi[int(output_dir[-6:])],
                            self.inter_func,
                            self.model_param,
                            B0,
                            bp,
                            etol,
                            ftol,
                            maxiter,
                            maxeval,
                        )
                    else:
                        fc = lammps_utils.make_lammps_equi(
                            "conf.lmp",
                            self.type_map,
                            self.inter_func,
                            self.model_param,
                            etol,
                            ftol,
                            maxiter,
                            maxeval,
                            True,
                        )
                elif [relax_pos, relax_shape, relax_vol] == [
                    True,
                    True,
                    False,
                ] and task_type == "eos":
                    task_param["cal_setting"]["relax_shape"] = False
                    fc = lammps_utils.make_lammps_equi(
                        "conf.lmp",
                        self.type_map,
                        self.inter_func,
                        self.model_param,
                        etol,
                        ftol,
                        maxiter,
                        maxeval,
                        False,
                    )
                elif [relax_pos, relax_shape, relax_vol] == [False, False, False]:
                    fc = lammps_utils.make_lammps_eval(
                        "conf.lmp", self.type_map, self.inter_func, self.model_param
                    )

                else:
                    raise RuntimeError("not supported calculation setting for LAMMPS")

            elif cal_type == "static":
                fc = lammps_utils.make_lammps_eval(
                    "conf.lmp", self.type_map, self.inter_func, self.model_param
                )

            else:
                raise RuntimeError("not supported calculation type for LAMMPS")

        dumpfn(task_param, os.path.join(output_dir, "task.json"), indent=4)

        in_lammps_not_link_list = ["eos"]
        if task_type not in in_lammps_not_link_list:
            with open(os.path.join(output_dir, "../in.lammps"), "w") as fp:
                fp.write(fc)
            cwd = os.getcwd()
            os.chdir(output_dir)
            if not (os.path.islink("in.lammps") or os.path.isfile("in.lammps")):
                os.symlink("../in.lammps", "in.lammps")
            else:
                os.remove("in.lammps")
                os.symlink("../in.lammps", "in.lammps")
            os.chdir(cwd)
        else:
            with open(os.path.join(output_dir, "in.lammps"), "w") as fp:
                fp.write(fc)

    def compute(self, output_dir):
        log_lammps = os.path.join(output_dir, "log.lammps")
        dump_lammps = os.path.join(output_dir, "dump.relax")
        if not os.path.isfile(log_lammps):
            warnings.warn("cannot find log.lammps in " + output_dir + " skip")
            return None
        if not os.path.isfile(dump_lammps):
            warnings.warn("cannot find dump.relax in " + output_dir + " skip")
            return None
        else:
            box = []
            coord = []
            vol = []
            energy = []
            force = []
            virial = []
            stress = []
            with open(dump_lammps, "r") as fin:
                dump = fin.read().split("\n")
            dumptime = []
            for idx, ii in enumerate(dump):
                if ii == "ITEM: TIMESTEP":
                    box.append([])
                    coord.append([])
                    force.append([])
                    dumptime.append(int(dump[idx + 1]))
                    natom = int(dump[idx + 3])
                    xlo_bound = float(dump[idx + 5].split()[0])
                    xhi_bound = float(dump[idx + 5].split()[1])
                    xy = float(dump[idx + 5].split()[2])
                    ylo_bound = float(dump[idx + 6].split()[0])
                    yhi_bound = float(dump[idx + 6].split()[1])
                    xz = float(dump[idx + 6].split()[2])
                    zlo = float(dump[idx + 7].split()[0])
                    zhi = float(dump[idx + 7].split()[1])
                    yz = float(dump[idx + 7].split()[2])
                    xx = (
                        xhi_bound
                        - max([0, xy, xz, xy + xz])
                        - (xlo_bound - min([0, xy, xz, xy + xz]))
                    )
                    yy = yhi_bound - max([0, yz]) - (ylo_bound - min([0, yz]))
                    zz = zhi - zlo
                    box[-1].append([xx, 0.0, 0.0])
                    box[-1].append([xy, yy, 0.0])
                    box[-1].append([xz, yz, zz])
                    vol.append(xx * yy * zz)
                    type_list = []
                    for jj in range(natom):
                        type_list.append(int(dump[idx + 9 + jj].split()[1]) - 1)
                        if "xs ys zs" in dump[idx + 8]:
                            a_x = (
                                float(dump[idx + 9 + jj].split()[2]) * xx
                                + float(dump[idx + 9 + jj].split()[3]) * xy
                                + float(dump[idx + 9 + jj].split()[4]) * xz
                            )
                            a_y = (
                                float(dump[idx + 9 + jj].split()[3]) * yy
                                + float(dump[idx + 9 + jj].split()[4]) * yz
                            )
                            a_z = float(dump[idx + 9 + jj].split()[4]) * zz
                        else:
                            a_x = float(dump[idx + 9 + jj].split()[2])
                            a_y = float(dump[idx + 9 + jj].split()[3])
                            a_z = float(dump[idx + 9 + jj].split()[4])
                        coord[-1].append([a_x, a_y, a_z])
                        fx = float(dump[idx + 9 + jj].split()[5])
                        fy = float(dump[idx + 9 + jj].split()[6])
                        fz = float(dump[idx + 9 + jj].split()[7])
                        force[-1].append([fx, fy, fz])

            with open(log_lammps, "r") as fp:
                if "Total wall time:" not in fp.read():
                    warnings.warn("lammps not finished " + log_lammps + " skip")
                    return None
                else:
                    fp.seek(0)
                    lines = fp.read().split("\n")
                    idid = -1
                    for ii in dumptime:
                        idid += 1
                        for jj in lines:
                            line = jj.split()
                            if len(line) and str(ii) == line[0]:
                                try:
                                    [float(kk) for kk in line]
                                except Exception:
                                    continue
                                stress.append([])
                                virial.append([])
                                energy.append(float(line[1]))
                                # virials = stress * vol * 1e5 *1e-30 * 1e19/1.6021766208
                                stress[-1].append(
                                    [
                                        float(line[2]) / 1000.0,
                                        float(line[5]) / 1000.0,
                                        float(line[6]) / 1000.0,
                                    ]
                                )
                                stress[-1].append(
                                    [
                                        float(line[5]) / 1000.0,
                                        float(line[3]) / 1000.0,
                                        float(line[7]) / 1000.0,
                                    ]
                                )
                                stress[-1].append(
                                    [
                                        float(line[6]) / 1000.0,
                                        float(line[7]) / 1000.0,
                                        float(line[4]) / 1000.0,
                                    ]
                                )
                                stress_to_virial = (
                                    vol[idid] * 1e5 * 1e-30 * 1e19 / 1.6021766208
                                )
                                virial[-1].append(
                                    [
                                        float(line[2]) * stress_to_virial,
                                        float(line[5]) * stress_to_virial,
                                        float(line[6]) * stress_to_virial,
                                    ]
                                )
                                virial[-1].append(
                                    [
                                        float(line[5]) * stress_to_virial,
                                        float(line[3]) * stress_to_virial,
                                        float(line[7]) * stress_to_virial,
                                    ]
                                )
                                virial[-1].append(
                                    [
                                        float(line[6]) * stress_to_virial,
                                        float(line[7]) * stress_to_virial,
                                        float(line[4]) * stress_to_virial,
                                    ]
                                )
                                break

            _tmp = self.type_map
            #dlog.debug(_tmp)
            type_map_list = lammps_utils.element_list(self.type_map)

            type_map_idx = list(range(len(type_map_list)))
            atom_numbs = []
            for ii in type_map_idx:
                count = 0
                for jj in type_list:
                    if jj == ii:
                        count += 1
                atom_numbs.append(count)

            # d_dump = dpdata.System(dump_lammps, fmt='lammps/dump', type_map=type_map_list)
            # d_dump.to('vasp/poscar', contcar, frame_idx=-1)

            result_dict = {
                "@module": "dpdata.system",
                "@class": "LabeledSystem",
                "data": {
                    "atom_numbs": atom_numbs,
                    "atom_names": type_map_list,
                    "atom_types": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "int64",
                        "data": type_list,
                    },
                    "orig": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "int64",
                        "data": [0, 0, 0],
                    },
                    "cells": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "float64",
                        "data": box,
                    },
                    "coords": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "float64",
                        "data": coord,
                    },
                    "energies": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "float64",
                        "data": energy,
                    },
                    "forces": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "float64",
                        "data": force,
                    },
                    "virials": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "float64",
                        "data": virial,
                    },
                    "stress": {
                        "@module": "numpy",
                        "@class": "array",
                        "dtype": "float64",
                        "data": stress,
                    },
                },
            }

            contcar = os.path.join(output_dir, "CONTCAR")
            dumpfn(result_dict, contcar, indent=4)
            d_dump = loadfn(contcar)
            d_dump.to("vasp/poscar", contcar, frame_idx=-1)

            return result_dict

    def forward_files(self, property_type="relaxation"):
        if self.inter_type == "meam":
            return ["conf.lmp", "in.lammps"] + list(map(os.path.basename, self.model))
        else:
            return ["conf.lmp", "in.lammps", os.path.basename(self.model)]

    def forward_common_files(self, property_type="relaxation"):
        if property_type not in ["eos"]:
            if self.inter_type == "meam":
                return ["in.lammps"] + list(map(os.path.basename, self.model))
            else:
                return ["in.lammps", os.path.basename(self.model)]
        else:
            if self.inter_type == "meam":
                return list(map(os.path.basename, self.model))
            else:
                return [os.path.basename(self.model)]

    def backward_files(self, property_type="relaxation"):
        return ["log.lammps", "outlog", "dump.relax"]




import os
import shutil
import sys
import unittest

from monty.serialization import loadfn

from apex.core.common_equi import make_equi, post_equi
from apex.core.calculator.lib import abacus_scf

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestEqui(unittest.TestCase):
    jdata = {
        "structures": ["confs/fcc-Al"],
        "interaction": {
            "type": "abacus",
            "incar": "abacus_input/INPUT",
            "potcar_prefix": "abacus_input",
            "potcars": {"Al": "Al_ONCV_PBE-1.0.upf"},
            "orb_files": {"Al": "Al_gga_9au_100Ry_4s4p1d.orb"},
        },
        "relaxation": {
            "cal_type": "relaxation",
            "cal_setting": {"input_prop": "abacus_input/INPUT"},
        },
    }

    def tearDown(self):
        if os.path.exists("confs/fcc-Al/relaxation"):
            shutil.rmtree("confs/fcc-Al/relaxation")

    def test_make_equi(self):
        confs = self.jdata["structures"]
        inter_param = self.jdata["interaction"]
        relax_param = self.jdata["relaxation"]
        make_equi(confs, inter_param, relax_param)

        target_path = "confs/fcc-Al/relaxation/relax_task"

        incar0 = abacus_scf.get_abacus_input_parameters(
            os.path.join("abacus_input", "INPUT")
        )
        incar1 = abacus_scf.get_abacus_input_parameters(
            os.path.join(target_path, "INPUT")
        )
        self.assertTrue(incar0 == incar1)

        with open(os.path.join("abacus_input", "Al_ONCV_PBE-1.0.upf")) as fp:
            pot0 = fp.read()
        with open(os.path.join(target_path, "pp_orb", "Al_ONCV_PBE-1.0.upf")) as fp:
            pot1 = fp.read()
        self.assertEqual(pot0, pot1)

        with open(os.path.join("abacus_input", "Al_gga_9au_100Ry_4s4p1d.orb")) as fp:
            pot0 = fp.read()
        with open(
            os.path.join(target_path, "pp_orb", "Al_gga_9au_100Ry_4s4p1d.orb")
        ) as fp:
            pot1 = fp.read()
        self.assertEqual(pot0, pot1)

        self.assertTrue(os.path.isfile(os.path.join(target_path, "KPT")))

        task_json_file = os.path.join(target_path, "task.json")
        self.assertTrue(os.path.isfile(task_json_file))
        task_json = loadfn(task_json_file)
        self.assertEqual(task_json, relax_param)

        inter_json_file = os.path.join(target_path, "inter.json")
        self.assertTrue(os.path.isfile(inter_json_file))
        inter_json = loadfn(inter_json_file)
        self.assertEqual(inter_json, inter_param)

        self.assertTrue(os.path.islink(os.path.join(target_path, "STRU")))

    def test_post_equi(self):
        confs = self.jdata["structures"]
        inter_param = self.jdata["interaction"]
        relax_param = self.jdata["relaxation"]
        target_path = "confs/fcc-Al/relaxation/relax_task"
        source_path = "equi/abacus"

        make_equi(confs, inter_param, relax_param)
        os.mkdir(os.path.join(target_path, "OUT.ABACUS"))
        shutil.copy(
            os.path.join(source_path, "INPUT"), os.path.join(target_path, "INPUT")
        )
        shutil.copy(
            os.path.join(source_path, "STRU"), os.path.join(target_path, "STRU")
        )
        shutil.copy(
            os.path.join(source_path, "running_cell-relax.log"),
            os.path.join(target_path, "OUT.ABACUS", "running_cell-relax.log"),
        )
        post_equi(confs, inter_param)

        result_json_file = os.path.join(target_path, "result.json")
        self.assertTrue(os.path.isfile(result_json_file))



import glob
import os
import shutil
import sys
import unittest

import dpdata
import numpy as np
from monty.serialization import loadfn
from pymatgen.io.vasp import Incar
from apex.core.property.Phonon import Phonon

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestPhonon(unittest.TestCase):
    def setUp(self):
        _jdata = {
            "structures": ["confs/std-bcc"],
            "interaction": {
                "type": "vasp",
                "potcar_prefix": "vasp_input",
                "potcars": {"Mo": "POTCAR_Mo"},
            },
            "properties": [
                {
                    "type": "phonon",
                    "skip": False,
                    "BAND": "0.0000 0.0000 0.5000  0.0000 0.0000 0.0000  0.5000 -0.5000 0.5000  0.25000 0.2500 0.2500  0 0 0",
                    "supercell_matrix": [2, 2, 2]
                },
            ],
        }

        self.equi_path = "confs/hp-Mo/relaxation/relax_task"
        self.source_path = "equi/vasp"
        self.target_path = "confs/hp-Mo/phonon_00"
        self.res_data = "output/phonon_00/result.json"
        self.ptr_data = "output/phonon_00/result.out"

        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)
        if not os.path.exists(self.target_path):
            os.makedirs(self.target_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata["properties"]

        self.phonon = Phonon(_jdata["properties"][0])

    def tearDown(self):
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)
        if os.path.exists(self.res_data):
            os.remove(self.res_data)
        if os.path.exists(self.ptr_data):
            os.remove(self.ptr_data)

    def test_task_type(self):
        self.assertEqual("phonon", self.phonon.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.phonon.task_param())

    def test_make_phonon_conf(self):
        if not os.path.exists(os.path.join(self.equi_path, "CONTCAR")):
            with self.assertRaises(RuntimeError):
                self.phonon.make_confs(self.target_path, self.equi_path)
        shutil.copy(
            os.path.join(self.source_path, "CONTCAR_Mo_bcc"),
            os.path.join(self.equi_path, "CONTCAR"),
        )
        task_list = self.phonon.make_confs(self.target_path, self.equi_path)
        dfm_dirs = glob.glob(os.path.join(self.target_path, "task.*"))
        self.assertEqual(len(dfm_dirs), 1)
        self.assertTrue(os.path.isfile(os.path.join(self.target_path, "phonopy_disp.yaml")))
        self.assertTrue(os.path.isfile(os.path.join(self.target_path, "task.000000/band.conf")))



#!/usr/bin/env python3
from typing import Type
from monty.serialization import loadfn
from dflow.python import OP
from dflow.python import upload_packages
from fpop.vasp import RunVasp
from fpop.abacus import RunAbacus
from apex.op.RunLAMMPS import RunLAMMPS

upload_packages.append(__file__)

MaxLength = 70


def return_prop_list(parameters: list) -> list:
    prop_list = []
    for ii in parameters:
        if ii.get('skip', False):
            continue
        if 'init_from_suffix' and 'output_suffix' in ii:
            # do_refine = True
            suffix = ii['output_suffix']
        elif 'reproduce' in ii and ii['reproduce']:
            # do_refine = False
            suffix = 'reprod'
        elif 'suffix' in ii and ii['suffix']:
            suffix = str(ii['suffix'])
        else:
            # do_refine = False
            suffix = '00'
        prop_list.append(ii['type'] + '_' + suffix)
    return prop_list


def get_flow_type(d: dict) -> str:
    if 'relaxation' in d and 'properties' not in d:
        flow_type = 'relax'
    elif 'properties' in d and 'relaxation' not in d:
        flow_type = 'props'
    elif 'relaxation' in d and 'properties' in d:
        flow_type = 'joint'
    else:
        raise RuntimeError('Can not recognize type of the input json file')
    return flow_type


def get_task_type(d: dict) -> (str, Type[OP]):
    interaction_type = d['interaction']['type']
    if interaction_type == 'vasp':
        task_type = 'vasp'
        run_op = RunVasp
    elif interaction_type == 'abacus':
        task_type = 'abacus'
        run_op = RunAbacus
    elif interaction_type in ['deepmd', 'eam_alloy']:
        task_type = 'lammps'
        run_op = RunLAMMPS
    else:
        raise RuntimeError(f'Unsupported interaction type: {interaction_type}')

    return task_type, run_op


def sepline(ch="-", sp="-", screen=False):
    r"""
    seperate the output by '-'
    """
    ch.center(MaxLength, sp)


def update_dict(d1: dict, d2: dict) -> None:
    if d2 is None:
        return None
    for k, v in d2.items():
        if isinstance(v, dict) and k in d1 and isinstance(d1[k], dict):
            update_dict(d1[k], v)
        else:
            d1[k] = v



import warnings
from monty.serialization import loadfn
from apex.core.common_equi import (make_equi, run_equi, post_equi)
from apex.core.common_prop import (make_property, run_property, post_property)
from apex.utils import get_flow_type, return_prop_list


def run_step(parameter, step, machine_file=None):
    print('-------Singel step local debug mode--------')
    param_dict = loadfn(parameter[0])
    # check input args
    json_type = get_flow_type(param_dict)
    mismatch1 = step in ['make_relax', 'run_relax', 'post_relax'] and json_type == 'props'
    mismatch2 = step in ['make_props', 'run_props', 'post_props'] and json_type == 'relax'
    if mismatch1 or mismatch2:
        raise RuntimeError(
            f'mismatched indication step ({step}) with type of json provided ({json_type})'
        )
    structures = param_dict["structures"]
    inter_parameter = param_dict["interaction"]
    if step in ['make_relax', 'run_relax', 'post_relax']:
        param = param_dict["relaxation"]
        if step == 'make_relax':
            print('Making relaxation tasks locally...')
            make_equi(structures, inter_parameter, param)
        elif step == 'run_relax':
            print('Run relaxation tasks locally...')
            if not machine_file:
                raise RuntimeWarning(
                    'Miss configuration file for dpdispatcher (indicate by optional args -c).'
                    'Jobs will be running on the local shell.'
                )
                mdata = {}
            else:
                mdata = loadfn(machine_file)
            run_equi(structures, inter_parameter, mdata)
        else:
            print('Posting relaxation results locally...')
            post_equi(structures, inter_parameter)

    elif step in ['make_props', 'run_props', 'post_props']:
        param = param_dict["properties"]
        if step == 'make_props':
            print('Making property tasks locally...')
            make_property(structures, inter_parameter, param)
        elif step == 'run_props':
            print('Run property tasks locally...')
            if not machine_file:
                raise RuntimeWarning(
                    'Miss configuration file for dpdispatcher (indicate by optional args -c).'
                    'Jobs will be running on the local shell.'
                )
                mdata = {}
            else:
                mdata = loadfn(machine_file)
            run_property(structures, inter_parameter, param, mdata)
        else:
            print('Posting property results locally...')
            post_property(structures, inter_parameter, param)



import glob
import json
import logging
import os
import re

import numpy as np
from monty.serialization import dumpfn, loadfn
from pymatgen.analysis.defects.generators import InterstitialGenerator
from pymatgen.core.structure import Structure

from apex.core.calculator.lib import abacus_utils
from apex.core.calculator.lib import lammps_utils
from apex.core.property.Property import Property
from apex.core.refine import make_refine
from apex.core.reproduce import make_repro, post_repro
from apex.core.structure import StructureInfo
from dflow.python import upload_packages
upload_packages.append(__file__)


class Interstitial(Property):
    def __init__(self, parameter, inter_param=None):
        parameter["reproduce"] = parameter.get("reproduce", False)
        self.reprod = parameter["reproduce"]
        if not self.reprod:
            if not ("init_from_suffix" in parameter and "output_suffix" in parameter):
                default_supercell = [1, 1, 1]
                parameter["supercell"] = parameter.get("supercell", default_supercell)
                self.supercell = parameter["supercell"]
                self.insert_ele = parameter["insert_ele"]
            parameter["cal_type"] = parameter.get("cal_type", "relaxation")
            self.cal_type = parameter["cal_type"]
            default_cal_setting = {
                "relax_pos": True,
                "relax_shape": True,
                "relax_vol": True,
            }
            if "cal_setting" not in parameter:
                parameter["cal_setting"] = default_cal_setting
            else:
                if "relax_pos" not in parameter["cal_setting"]:
                    parameter["cal_setting"]["relax_pos"] = default_cal_setting[
                        "relax_pos"
                    ]
                if "relax_shape" not in parameter["cal_setting"]:
                    parameter["cal_setting"]["relax_shape"] = default_cal_setting[
                        "relax_shape"
                    ]
                if "relax_vol" not in parameter["cal_setting"]:
                    parameter["cal_setting"]["relax_vol"] = default_cal_setting[
                        "relax_vol"
                    ]
            self.cal_setting = parameter["cal_setting"]
        else:
            parameter["cal_type"] = "static"
            self.cal_type = parameter["cal_type"]
            default_cal_setting = {
                "relax_pos": False,
                "relax_shape": False,
                "relax_vol": False,
            }
            if "cal_setting" not in parameter:
                parameter["cal_setting"] = default_cal_setting
            else:
                if "relax_pos" not in parameter["cal_setting"]:
                    parameter["cal_setting"]["relax_pos"] = default_cal_setting[
                        "relax_pos"
                    ]
                if "relax_shape" not in parameter["cal_setting"]:
                    parameter["cal_setting"]["relax_shape"] = default_cal_setting[
                        "relax_shape"
                    ]
                if "relax_vol" not in parameter["cal_setting"]:
                    parameter["cal_setting"]["relax_vol"] = default_cal_setting[
                        "relax_vol"
                    ]
            self.cal_setting = parameter["cal_setting"]
            parameter["init_from_suffix"] = parameter.get("init_from_suffix", "00")
            self.init_from_suffix = parameter["init_from_suffix"]
        self.parameter = parameter
        self.inter_param = inter_param if inter_param != None else {"type": "vasp"}

    def make_confs(self, path_to_work, path_to_equi, refine=False):
        self.path_to_work = os.path.abspath(path_to_work)
        if os.path.exists(path_to_work):
            logging.warning("%s already exists" % path_to_work)
            # dlog.warning("%s already exists" % path_to_work)
        else:
            os.makedirs(path_to_work)
        path_to_equi = os.path.abspath(path_to_equi)

        if "start_confs_path" in self.parameter and os.path.exists(
            self.parameter["start_confs_path"]
        ):
            init_path_list = glob.glob(
                os.path.join(self.parameter["start_confs_path"], "*")
            )
            struct_init_name_list = []
            for ii in init_path_list:
                struct_init_name_list.append(ii.split("/")[-1])
            struct_output_name = self.path_to_work.split("/")[-2]
            assert struct_output_name in struct_init_name_list
            path_to_equi = os.path.abspath(
                os.path.join(
                    self.parameter["start_confs_path"],
                    struct_output_name,
                    "relaxation",
                    "relax_task",
                )
            )

        self.task_list = []
        cwd = os.getcwd()

        if self.reprod:
            print("interstitial reproduce starts")
            if "init_data_path" not in self.parameter:
                raise RuntimeError("please provide the initial data path to reproduce")
            init_data_path = os.path.abspath(self.parameter["init_data_path"])
            self.task_list = make_repro(
                self.inter_param,
                init_data_path,
                self.init_from_suffix,
                self.path_to_work,
                self.parameter.get("reprod_last_frame", False),
            )
            os.chdir(cwd)

        else:
            if refine:
                print("interstitial refine starts")
                self.task_list = make_refine(
                    self.parameter["init_from_suffix"],
                    self.parameter["output_suffix"],
                    self.path_to_work,
                )

                init_from_path = re.sub(
                    self.parameter["output_suffix"][::-1],
                    self.parameter["init_from_suffix"][::-1],
                    self.path_to_work[::-1],
                    count=1,
                )[::-1]
                task_list_basename = list(map(os.path.basename, self.task_list))

                os.chdir(self.path_to_work)
                if os.path.isfile("element.out"):
                    os.remove("element.out")
                if os.path.islink("element.out"):
                    os.remove("element.out")
                os.symlink(
                    os.path.relpath(os.path.join(init_from_path, "element.out")),
                    "element.out",
                )
                os.chdir(cwd)

                for ii in task_list_basename:
                    init_from_task = os.path.join(init_from_path, ii)
                    output_task = os.path.join(self.path_to_work, ii)
                    os.chdir(output_task)
                    if os.path.isfile("supercell.json"):
                        os.remove("supercell.json")
                    if os.path.islink("supercell.json"):
                        os.remove("supercell.json")
                    os.symlink(
                        os.path.relpath(os.path.join(init_from_task, "supercell.json")),
                        "supercell.json",
                    )
                os.chdir(cwd)

            else:
                if self.inter_param["type"] == "abacus":
                    CONTCAR = abacus_utils.final_stru(path_to_equi)
                    POSCAR = "STRU"
                else:
                    CONTCAR = "CONTCAR"
                    POSCAR = "POSCAR"

                equi_contcar = os.path.join(path_to_equi, CONTCAR)
                if not os.path.exists(equi_contcar):
                    raise RuntimeError("please do relaxation first")

                if self.inter_param["type"] == "abacus":
                    ss = abacus_utils.stru2Structure(equi_contcar)
                else:
                    ss = Structure.from_file(equi_contcar)

                # get structure type
                os.chdir(self.path_to_work)
                ss.to("POSCAR", "POSCAR")
                st = StructureInfo(ss)
                self.structure_type = st.lattice_structure
                os.chdir(cwd)

                # gen defects
                dss = []
                self.insert_element_task = os.path.join(self.path_to_work, "element.out")
                if os.path.isfile(self.insert_element_task):
                    os.remove(self.insert_element_task)

                for ii in self.insert_ele:
                    pre_vds = InterstitialGenerator()
                    vds = pre_vds.generate(ss, {ii: [[0.1, 0.1, 0.1]]})
                    for jj in vds:
                        temp = jj.get_supercell_structure(
                            sc_mat=np.diag(self.supercell, k=0)
                        )
                        smallest_distance = list(set(temp.distance_matrix.ravel()))[1]
                        if (
                            "conf_filters" in self.parameter
                            and "min_dist" in self.parameter["conf_filters"]
                        ):
                            min_dist = self.parameter["conf_filters"]["min_dist"]
                            if smallest_distance >= min_dist:
                                dss.append(temp)
                                with open(self.insert_element_task, "a+") as fout:
                                    print(ii, file=fout)
                        else:
                            dss.append(temp)
                            with open(self.insert_element_task, "a+") as fout:
                                print(ii, file=fout)
                #            dss.append(jj.generate_defect_structure(self.supercell))
                        self.dss = dss

                print(
                    "gen interstitial with supercell "
                    + str(self.supercell)
                    + " with element "
                    + str(self.insert_ele)
                )
                os.chdir(self.path_to_work)
                if os.path.isfile(POSCAR):
                    os.remove(POSCAR)
                if os.path.islink(POSCAR):
                    os.remove(POSCAR)
                os.symlink(os.path.relpath(equi_contcar), POSCAR)
                #           task_poscar = os.path.join(output, 'POSCAR')

                for ii in range(len(dss)):
                    output_task = os.path.join(self.path_to_work, "task.%06d" % ii)
                    os.makedirs(output_task, exist_ok=True)
                    os.chdir(output_task)
                    for jj in [
                        "INCAR",
                        "POTCAR",
                        "POSCAR",
                        "conf.lmp",
                        "in.lammps",
                        "STRU",
                    ]:
                        if os.path.exists(jj):
                            os.remove(jj)
                    self.task_list.append(output_task)
                    dss[ii].to("POSCAR", "POSCAR")
                    # np.savetxt('supercell.out', self.supercell, fmt='%d')
                    dumpfn(self.supercell, "supercell.json")
                os.chdir(cwd)

                super_size = (
                        self.supercell[0] * self.supercell[1] * self.supercell[2]
                )
                num_atom = super_size * 2
                #chl = -num_atom - 2
                os.chdir(self.path_to_work)

                if not os.path.isfile("task.000000/POSCAR"):
                    raise RuntimeError("need task.000000 structure as reference")

                with open('POSCAR', "r") as fin:
                    fin.readline()
                    scale = float(fin.readline().split()[0])
                    self.latt_param = float(fin.readline().split()[0])
                    self.latt_param *= scale

                with open("task.000000/POSCAR", "r") as fin:
                    self.pos_line = fin.read().split("\n")


                self.super_latt_param = float(self.pos_line[2].split()[0])
                self.unit_frac = self.latt_param / self.super_latt_param

                for idx, ii in enumerate(self.pos_line):
                    ss = ii.split()
                    if len(ss) > 3:
                        if (
                                abs(self.unit_frac * 0.1 - float(ss[0])) < 1e-5
                                and abs(self.unit_frac * 0.1 - float(ss[1])) < 1e-5
                                and abs(self.unit_frac * 0.1 - float(ss[2])) < 1e-5
                        ):
                            chl = idx

                os.chdir(cwd)

                # specify interstitial structures
                if self.structure_type == 'bcc':
                    for idx, ii in enumerate(self.pos_line):
                        ss = ii.split()
                        if len(ss) > 3:
                            if (
                                    abs(self.unit_frac * 0.5 - float(ss[0])) < 1e-5
                                    and abs(self.unit_frac * 0.5 - float(ss[1])) < 1e-5
                                    and abs(self.unit_frac * 0.5 - float(ss[2])) < 1e-5
                            ):
                                center = idx
                    bcc_interstital_dict = {
                        'tetrahedral':   {chl: [0.25, 0.5, 0]},
                        'octahedral':    {chl: [0.5, 0.5, 0]},
                        'crowdion':      {chl: [0.25, 0.25, 0]},
                        '<111>dumbbell': {chl: [1/3, 1/3, 1/3],
                                          center: [2/3, 2/3, 2/3]},
                        '<110>dumbbell': {chl: [1/4, 3/4, 1/2],
                                          center: [3/4, 1/4, 1/2]},
                        '<100>dumbbell': {chl: [1/2, 1/2, 1/6],
                                          center: [1/2, 1/2, 5/6]}
                    }
                    total_task = self.__gen_tasks(bcc_interstital_dict)

                elif self.structure_type == 'fcc':
                    for idx, ii in enumerate(self.pos_line):
                        ss = ii.split()
                        if len(ss) > 3:
                            if (
                                    abs(self.unit_frac * 1 - float(ss[0])) < 1e-5
                                    and abs(self.unit_frac * 0.5 - float(ss[1])) < 1e-5
                                    and abs(self.unit_frac * 0.5 - float(ss[2])) < 1e-5
                            ):
                                face = idx

                            if (
                                    abs(self.unit_frac * 1 - float(ss[0])) < 1e-5
                                    and abs(self.unit_frac * 1 - float(ss[1])) < 1e-5
                                    and abs(self.unit_frac * 1 - float(ss[2])) < 1e-5
                            ):
                                corner = idx

                    fcc_interstital_dict = {
                        'tetrahedral':      {chl: [0.75, 0.25, 0.25]},
                        'octahedral':       {chl: [1, 0, 0.5]},
                        'crowdion':         {chl: [1, 0.25, 0.25]},
                        '<111>dumbbell':    {
                            chl: [1-0.3/np.sqrt(3),
                                    1-0.3/np.sqrt(3),
                                    1-0.3/np.sqrt(3)],
                            corner: [0.3/np.sqrt(3),
                                    0.3/np.sqrt(3),
                                    0.3/np.sqrt(3)]
                        },
                        '<110>dumbbell': {
                            chl: [1,
                                    0.5+(0.3/np.sqrt(2)),
                                    0.5+(0.3/np.sqrt(2))],
                            face: [1,
                                    0.5-(0.3/np.sqrt(2)),
                                    0.5-(0.3/np.sqrt(2))]
                        },
                        '<100>dumbbell': {
                            chl: [1, 0.2, 0.5],
                            face: [1, 0.8, 0.5]
                        },
                    }
                    total_task = self.__gen_tasks(fcc_interstital_dict)

                elif self.structure_type == 'hcp':
                    pass
                else:
                    total_task = len(dss)

                if self.inter_param["type"] == "abacus":
                    for ii in range(total_task):
                        output_task = os.path.join(self.path_to_work, "task.%06d" % ii)
                        os.chdir(output_task)
                        abacus_utils.poscar2stru("POSCAR", self.inter_param, "STRU")
                        os.remove("POSCAR")
                    os.chdir(cwd)

        return self.task_list


    def __gen_tasks(self, interstitial_dict):
        cwd = os.getcwd()
        for ii, (type_str, adjust_dict) in enumerate(interstitial_dict.items()):
            output_task = os.path.join(
                self.path_to_work, "task.%06d" % (len(self.dss) + ii)
            )
            os.makedirs(output_task, exist_ok=True)
            os.chdir(output_task)
            self.task_list.append(output_task)
            # adjust atom positions in POSCAR
            with open(self.insert_element_task, "a+") as fout:
                print(self.insert_ele[0], file=fout)
            dumpfn(self.supercell, "supercell.json")
            dumpfn(type_str, 'interstitial_type.json')
            new_pos_line = self.pos_line.copy()
            for line, pos in adjust_dict.items():
                new_pos_line[line] = (
                        "%.6f" % float(self.unit_frac * pos[0])
                        + " "
                        + "%.6f" % float(self.unit_frac * pos[1])
                        + " "
                        + "%.6f" % float(self.unit_frac * pos[2])
                        + " "
                        + self.insert_ele[0]
                )
            with open("POSCAR", "w+") as fout:
                for ii in new_pos_line:
                    print(ii, file=fout)
            print(f"gen {type_str}")
            os.chdir(cwd)

        total_task = len(self.dss) + len(interstitial_dict)

        return total_task

    def post_process(self, task_list):
        if True:
            fin1 = open(os.path.join(task_list[0], "..", "element.out"), "r")
            for ii in task_list:
                conf = os.path.join(ii, "conf.lmp")
                inter = os.path.join(ii, "inter.json")
                insert_ele = fin1.readline().split()[0]
                if os.path.isfile(conf):
                    with open(conf, "r") as fin2:
                        conf_line = fin2.read().split("\n")
                        insert_line = conf_line[-2]
                    type_map = loadfn(inter)["type_map"]
                    type_map_list = lammps_utils.element_list(type_map)
                    if int(insert_line.split()[1]) > len(type_map_list):
                        type_num = type_map[insert_ele] + 1
                        conf_line[2] = str(len(type_map_list)) + " atom types"
                        conf_line[-2] = (
                            "%6.d" % int(insert_line.split()[0])
                            + "%7.d" % type_num
                            + "%16.10f" % float(insert_line.split()[2])
                            + "%16.10f" % float(insert_line.split()[3])
                            + "%16.10f" % float(insert_line.split()[4])
                        )
                        with open(conf, "w+") as fout:
                            for jj in conf_line:
                                print(jj, file=fout)
            fin1.close()

    def task_type(self):
        return self.parameter["type"]

    def task_param(self):
        return self.parameter

    def _compute_lower(self, output_file, all_tasks, all_res):
        output_file = os.path.abspath(output_file)
        res_data = {}
        ptr_data = os.path.dirname(output_file) + "\n"

        if not self.reprod:
            with open(
                os.path.join(os.path.dirname(output_file), "element.out"), "r"
            ) as fin:
                fc = fin.read().split("\n")
            ptr_data += "Insert_ele-Struct: Inter_E(eV)  E(eV) equi_E(eV)\n"
            idid = -1
            for ii in all_tasks:
                idid += 1
                structure_dir = os.path.basename(ii)
                task_result = loadfn(all_res[idid])
                natoms = task_result["atom_numbs"][0]
                equi_path = os.path.abspath(
                    os.path.join(
                        os.path.dirname(output_file), "../relaxation/relax_task"
                    )
                )
                equi_result = loadfn(os.path.join(equi_path, "result.json"))
                equi_epa = equi_result["energies"][-1] / equi_result["atom_numbs"][0]
                evac = task_result["energies"][-1] - equi_epa * natoms

                supercell_index = loadfn(os.path.join(ii, "supercell.json"))
                # insert_ele = loadfn(os.path.join(ii, 'task.json'))['insert_ele'][0]
                insert_ele = fc[idid]
                ptr_data += "%s: %7.3f  %7.3f %7.3f \n" % (
                    insert_ele + "-" + str(supercell_index) + "-" + structure_dir,
                    evac,
                    task_result["energies"][-1],
                    equi_epa * natoms,
                )
                res_data[
                    insert_ele + "-" + str(supercell_index) + "-" + structure_dir
                ] = [evac, task_result["energies"][-1], equi_epa * natoms]

        else:
            if "init_data_path" not in self.parameter:
                raise RuntimeError("please provide the initial data path to reproduce")
            init_data_path = os.path.abspath(self.parameter["init_data_path"])
            res_data, ptr_data = post_repro(
                init_data_path,
                self.parameter["init_from_suffix"],
                all_tasks,
                ptr_data,
                self.parameter.get("reprod_last_frame", False),
            )

        with open(output_file, "w") as fp:
            json.dump(res_data, fp, indent=4)

        return res_data, ptr_data



import glob
import os
import shutil
import sys
import unittest

from monty.serialization import loadfn
from pymatgen.core import Structure
from pymatgen.core.surface import SlabGenerator
from pymatgen.io.vasp import Incar

from apex.core.property.Surface import Surface

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestSurface(unittest.TestCase):
    def setUp(self):
        _jdata = {
            "structures": ["confs/mp-141"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR.rlx",
                "potcar_prefix": ".",
                "potcars": {"Yb": "vasp_input/POTCAR"},
            },
            "properties": [
                {
                    "type": "surface",
                    "min_slab_size": 10,
                    "min_vacuum_size": 11,
                    "pert_xz": 0.01,
                    "max_miller": 1,
                    "cal_type": "relaxation",
                }
            ],
        }

        self.equi_path = "confs/mp-141/relaxation/relax_task"
        self.source_path = "equi/vasp"
        self.target_path = "confs/mp-141/surface_00"
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata["properties"]

        self.surface = Surface(_jdata["properties"][0])

    def tearDown(self):
        if os.path.exists(os.path.abspath(os.path.join(self.equi_path, ".."))):
            shutil.rmtree(os.path.abspath(os.path.join(self.equi_path, "..")))
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

    def test_task_type(self):
        self.assertEqual("surface", self.surface.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.surface.task_param())

    def test_make_confs_0(self):
        if not os.path.exists(os.path.join(self.equi_path, "CONTCAR")):
            with self.assertRaises(RuntimeError):
                self.surface.make_confs(self.target_path, self.equi_path)
        shutil.copy(
            os.path.join(self.source_path, "mp-141.vasp"),
            os.path.join(self.equi_path, "CONTCAR"),
        )
        task_list = self.surface.make_confs(self.target_path, self.equi_path)
        self.assertEqual(len(task_list), 7)
        dfm_dirs = glob.glob(os.path.join(self.target_path, "task.*"))

        incar0 = Incar.from_file(os.path.join("vasp_input", "INCAR.rlx"))
        incar0["ISIF"] = 4

        self.assertEqual(
            os.path.realpath(os.path.join(self.equi_path, "CONTCAR")),
            os.path.realpath(os.path.join(self.target_path, "POSCAR")),
        )
        ref_st = Structure.from_file(os.path.join(self.target_path, "POSCAR"))
        dfm_dirs.sort()
        for ii in dfm_dirs:
            st_file = os.path.join(ii, "POSCAR")
            self.assertTrue(os.path.isfile(st_file))
            st0 = Structure.from_file(st_file)
            st1_file = os.path.join(ii, "POSCAR.tmp")
            self.assertTrue(os.path.isfile(st1_file))
            st1 = Structure.from_file(st1_file)
            miller_json_file = os.path.join(ii, "miller.json")
            self.assertTrue(os.path.isfile(miller_json_file))
            miller_json = loadfn(miller_json_file)
            sl = SlabGenerator(
                ref_st,
                miller_json,
                self.prop_param[0]["min_slab_size"],
                self.prop_param[0]["min_vacuum_size"],
            )
            slb = sl.get_slab()
            st2 = Structure(slb.lattice, slb.species, slb.frac_coords)
            self.assertEqual(len(st1), len(st2))



import os
import sys
import unittest

from monty.serialization import dumpfn, loadfn
from pymatgen.analysis.structure_matcher import StructureMatcher
from pymatgen.core import Structure

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"
from apex.core.mpdb import get_structure

from .context import make_kspacing_kpoints, setUpModule

try:
    os.environ["MAPI_KEY"]
    exist_key = True
except Exception:
    exist_key = False


def fit(struct0, struct1):
    m = StructureMatcher()
    if m.fit(struct0, struct1):
        return True
    return False


@unittest.skipIf(not exist_key, "skip mpdb")
class TestMpdb(unittest.TestCase):
    def setUp(self):
        if "MAPI_KEY" in os.environ:
            self.key = os.environ["MAPI_KEY"]
        else:
            self.key = None
        self.mpid = "mp-141"
        self.st_file = self.mpid + ".vasp"
        self.st0_file = os.path.join("confs/", self.mpid, self.mpid + ".cif")

    def tearDown(self):
        if os.path.exists(self.st_file):
            os.remove(self.st_file)

    def test_get_structure(self):
        st1 = get_structure(self.mpid)
        st0 = Structure.from_file(self.st0_file)
        self.assertTrue(fit(st0, st1))



import glob
import json
import os
import shutil
import sys
import unittest

import dpdata
import numpy as np
from monty.serialization import dumpfn, loadfn

from apex.core.calculator.Lammps import Lammps
from apex.core.calculator.lib.lammps_utils import inter_deepmd

#from .context import make_kspacing_kpoints, setUpModule

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestLammps(unittest.TestCase):
    def setUp(self):
        self.jdata = {
            "structures": ["confs/std-fcc"],
            "interaction": {
                "type": "deepmd",
                "model": "lammps_input/frozen_model.pb",
                "deepmd_version": "1.1.0",
                "type_map": {"Al": 0},
            },
            "relaxation": {
                "cal_type": "relaxation",
                "cal_setting": {
                    "relax_pos": True,
                    "relax_shape": True,
                    "relax_vol": True,
                },
            },
        }

        self.equi_path = "confs/std-fcc/relaxation/relax_task"
        self.source_path = "equi/lammps"

        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        if not os.path.isfile(os.path.join(self.equi_path, "POSCAR")):
            shutil.copy(
                os.path.join(self.source_path, "Al-fcc.vasp"),
                os.path.join("confs/std-fcc", "POSCAR"),
            )

        self.confs = self.jdata["structures"]
        self.inter_param = self.jdata["interaction"]
        self.relax_param = self.jdata["relaxation"]
        self.Lammps = Lammps(
            self.inter_param, os.path.join(self.source_path, "Al-fcc.vasp")
        )

    def tearDown(self):
        if os.path.exists("confs/std-fcc/relaxation"):
            shutil.rmtree("confs/std-fcc/relaxation")

    def test_set_inter_type_func(self):
        self.Lammps.set_inter_type_func()
        self.assertEqual(inter_deepmd, self.Lammps.inter_func)

    def test_set_model_param(self):
        self.Lammps.set_model_param()
        model_param = {
            "model_name": ["frozen_model.pb"],
            "param_type": {"Al": 0},
            "deepmd_version": "1.1.0",
        }
        self.assertEqual(model_param, self.Lammps.model_param)

    def test_make_potential_files(self):
        cwd = os.getcwd()
        abs_equi_path = os.path.abspath(self.equi_path)
        self.Lammps.make_potential_files(abs_equi_path)
        self.assertTrue(os.path.islink(os.path.join(self.equi_path, "frozen_model.pb")))
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, "inter.json")))
        ret = loadfn(os.path.join(self.equi_path, "inter.json"))
        self.assertEqual(self.inter_param, ret)
        os.chdir(cwd)

    def test_make_input_file(self):
        cwd = os.getcwd()
        abs_equi_path = os.path.abspath("confs/std-fcc/relaxation/relax_task")
        shutil.copy(
            os.path.join("confs/std-fcc", "POSCAR"),
            os.path.join(self.equi_path, "POSCAR"),
        )
        self.Lammps.make_input_file(abs_equi_path, "relaxation", self.relax_param)
        self.assertTrue(os.path.isfile(os.path.join(abs_equi_path, "conf.lmp")))
        self.assertTrue(os.path.islink(os.path.join(abs_equi_path, "in.lammps")))
        self.assertTrue(os.path.isfile(os.path.join(abs_equi_path, "task.json")))

    def test_forward_common_files(self):
        fc_files = ["in.lammps", "frozen_model.pb"]
        self.assertEqual(self.Lammps.forward_common_files(), fc_files)

    def test_backward_files(self):
        backward_files = ["log.lammps", "outlog", "dump.relax"]
        self.assertEqual(self.Lammps.backward_files(), backward_files)



#!/usr/bin/python3

import numpy as np
from dflow.python import upload_packages
upload_packages.append(__file__)
# from lib.vasp import system_from_poscar


def _make_pwscf_01_runctrl(sys_data, ecut, ediff, smearing, degauss):
    tot_natoms = sum(sys_data["atom_numbs"])
    ntypes = len(sys_data["atom_names"])
    ret = ""
    ret += "&control\n"
    ret += "calculation='scf',\n"
    ret += "restart_mode='from_scratch',\n"
    ret += "pseudo_dir='./',\n"
    ret += "outdir='./OUT',\n"
    ret += "tprnfor = .TRUE.,\n"
    ret += "tstress = .TRUE.,\n"
    ret += "disk_io = 'none',\n"
    ret += "/\n"
    ret += "&system\n"
    ret += "ibrav= 0,\n"
    ret += "nat  = %d,\n" % tot_natoms
    ret += "ntyp = %d,\n" % ntypes
    ret += "vdw_corr = 'TS',\n"
    ret += "ecutwfc = %f,\n" % ecut
    ret += "ts_vdw_econv_thr=%e,\n" % ediff
    ret += "nosym = .TRUE.,\n"
    if degauss is not None:
        ret += "degauss = %f,\n" % degauss
    if smearing is not None:
        ret += "smearing = '%s',\n" % (smearing.lower())
    ret += "/\n"
    ret += "&electrons\n"
    ret += "conv_thr = %e,\n" % ediff
    ret += "/\n"
    return ret


def _make_pwscf_02_species(sys_data, pps):
    atom_names = sys_data["atom_names"]
    if "atom_masses" in sys_data:
        atom_masses = sys_data["atom_masses"]
    else:
        atom_masses = [1 for ii in atom_names]
    ret = ""
    ret += "ATOMIC_SPECIES\n"
    ntypes = len(atom_names)
    assert ntypes == len(atom_names)
    assert ntypes == len(atom_masses)
    assert ntypes == len(pps)
    for ii in range(ntypes):
        ret += "%s %d %s\n" % (atom_names[ii], atom_masses[ii], pps[ii])
    return ret


def _make_pwscf_03_config(sys_data):
    cell = sys_data["cell"]
    cell = np.reshape(cell, [3, 3])
    coordinates = sys_data["coordinates"]
    atom_names = sys_data["atom_names"]
    atom_numbs = sys_data["atom_numbs"]
    ntypes = len(atom_names)
    ret = ""
    ret += "CELL_PARAMETERS { angstrom }\n"
    for ii in range(3):
        for jj in range(3):
            ret += "%f " % cell[ii][jj]
        ret += "\n"
    ret += "\n"
    ret += "ATOMIC_POSITIONS { angstrom }\n"
    cc = 0
    for ii in range(ntypes):
        for jj in range(atom_numbs[ii]):
            ret += "%s %f %f %f\n" % (
                atom_names[ii],
                coordinates[cc][0],
                coordinates[cc][1],
                coordinates[cc][2],
            )
            cc += 1
    return ret


def _kshift(nkpt):
    if (nkpt // 2) * 2 == nkpt:
        return 1
    else:
        return 0


def _make_pwscf_04_kpoints(sys_data, kspacing):
    cell = sys_data["cell"]
    cell = np.reshape(cell, [3, 3])
    rcell = np.linalg.inv(cell)
    rcell = rcell.T
    kpoints = [
        (np.ceil(2 * np.pi * np.linalg.norm(ii) / kspacing).astype(int)) for ii in rcell
    ]
    ret = ""
    ret += "K_POINTS { automatic }\n"
    for ii in range(3):
        ret += "%d " % kpoints[ii]
    for ii in range(3):
        ret += "%d " % _kshift(kpoints[ii])
    ret += "\n"
    return ret


def _make_smearing(fp_params):
    smearing = None
    degauss = None
    if "smearing" in fp_params:
        smearing = (fp_params["smearing"]).lower()
    if "sigma" in fp_params:
        degauss = fp_params["sigma"]
    if (smearing is not None) and (smearing.split(":")[0] == "mp"):
        smearing = "mp"
    if not (smearing in [None, "gauss", "mp", "fd"]):
        raise RuntimeError("unknow smearing method " + smearing)
    return smearing, degauss


def make_pwscf_input(sys_data, fp_pp_files, fp_params):
    ecut = fp_params["ecut"]
    ediff = fp_params["ediff"]
    kspacing = fp_params["kspacing"]
    smearing, degauss = _make_smearing(fp_params)
    ret = ""
    ret += _make_pwscf_01_runctrl(sys_data, ecut, ediff, smearing, degauss)
    ret += "\n"
    ret += _make_pwscf_02_species(sys_data, fp_pp_files)
    ret += "\n"
    ret += _make_pwscf_03_config(sys_data)
    ret += "\n"
    ret += _make_pwscf_04_kpoints(sys_data, kspacing)
    ret += "\n"
    return ret


# sys_data = system_from_poscar('POSCAR')
# ret = ""
# ret += _make_pwscf_01_runctrl(sys_data, 20, 1e-8)
# ret += "\n"
# ret += _make_pwscf_02_species(sys_data, ['../pp/C_HSCV_PBE-1.0.UPF', '../H_HSCV_PBE-1.0.UPF', '../N_HSCV_PBE-1.0.UPF'])
# ret += "\n"
# ret += _make_pwscf_03_config(sys_data)
# ret += "\n"
# ret += _make_pwscf_04_kpoints(sys_data, 0.6)
# ret += "\n"

# open('input', 'w').write(ret)



import os
import logging

from dpdata import LabeledSystem
from monty.serialization import dumpfn

from apex.core.calculator.lib import abacus_utils, abacus_scf
#from dpgen import dlog
from apex.core.calculator.Task import Task
from apex.utils import sepline
from dflow.python import upload_packages
upload_packages.append(__file__)


class ABACUS(Task):
    def __init__(self, inter_parameter, path_to_poscar):
        self.inter = inter_parameter
        self.inter_type = inter_parameter["type"]
        self.incar = inter_parameter.get("incar", {})
        self.potcar_prefix = inter_parameter.get("potcar_prefix", "")
        self.potcars = inter_parameter.get("potcars", None)
        self.orbfile = inter_parameter.get("orb_files", None)
        self.deepks = inter_parameter.get("deepks_desc", None)
        self.path_to_poscar = path_to_poscar
        self.if_define_orb_file = False if self.orbfile == None else True

    def make_potential_files(self, output_dir):
        stru = os.path.abspath(os.path.join(output_dir, "STRU"))
        if not os.path.isfile(stru):
            raise FileNotFoundError("No file %s" % stru)
        stru_data = abacus_scf.get_abacus_STRU(stru)
        atom_names = stru_data["atom_names"]
        orb_files = stru_data["orb_files"]
        pp_files = stru_data["pp_files"]
        dpks_descriptor = stru_data["dpks_descriptor"]

        if os.path.islink(os.path.join(output_dir, "STRU")):
            stru_path, tmpf = os.path.split(
                os.readlink(os.path.join(output_dir, "STRU"))
            )
        else:
            stru_path = output_dir

        if pp_files == None:
            raise RuntimeError("No pseudopotential information in STRU file")

        pp_dir = os.path.abspath(self.potcar_prefix)
        cwd = os.getcwd()
        os.chdir(output_dir)
        if not os.path.isdir("./pp_orb"):
            os.mkdir("./pp_orb")
        for i in range(len(atom_names)):
            pp_orb_file = [[pp_files[i], self.potcars]]
            if orb_files != None:
                pp_orb_file.append([orb_files[i], self.orbfile])
            elif self.orbfile != None:
                assert atom_names[i] in self.orbfile, (
                    "orb_file of %s is not defined" % atom_names[i]
                )
                pp_orb_file.append([self.orbfile[atom_names[i]], self.orbfile])

            if dpks_descriptor != None:
                pp_orb_file.append([dpks_descriptor[i], self.deepks])
            elif self.deepks != None:
                pp_orb_file.append([self.deepks, self.deepks])

            for tmpf, tmpdict in pp_orb_file:
                atom = atom_names[i]
                if os.path.isfile(os.path.join(stru_path, tmpf)):
                    linked_file = os.path.join(stru_path, tmpf)
                elif tmpdict != None and os.path.isfile(
                    os.path.join(pp_dir, tmpdict[atom])
                ):
                    linked_file = os.path.join(pp_dir, tmpdict[atom])
                else:
                    raise RuntimeError("Can not find file %s" % tmpf.split("/")[-1])
                target_file = os.path.join("./pp_orb/", tmpf.split("/")[-1])
                if os.path.isfile(target_file):
                    os.remove(target_file)
                os.symlink(linked_file, target_file)

        os.chdir(cwd)

        dumpfn(self.inter, os.path.join(output_dir, "inter.json"), indent=4)

    def modify_input(self, incar, x, y):
        if x in incar and incar[x] != y:
            logging.info("setting %s to %s" % (x, y))
        incar[x] = y

    def make_input_file(self, output_dir, task_type, task_param):
        sepline(ch=output_dir)
        dumpfn(task_param, os.path.join(output_dir, "task.json"), indent=4)

        assert os.path.exists(self.incar), "no INPUT file for relaxation"
        relax_incar_path = os.path.abspath(self.incar)
        incar_relax = abacus_scf.get_abacus_input_parameters(relax_incar_path)

        # deal with relaxation
        prop_type = task_param.get("type", "relaxation")
        cal_type = task_param["cal_type"]
        cal_setting = task_param["cal_setting"]

        # user input INCAR for apex calculation
        if "input_prop" in cal_setting and os.path.isfile(cal_setting["input_prop"]):
            incar_prop = os.path.abspath(cal_setting["input_prop"])
            incar = abacus_scf.get_abacus_input_parameters(incar_prop)
            logging.info(
                "Detected 'input_prop' in 'relaxation', use %s as INPUT, and ignore 'cal_setting'"
                % incar_prop
            )

        # revise INCAR based on the INCAR provided in the "interaction"
        else:
            incar = incar_relax
            for key in cal_setting:
                if key in ["relax_pos", "relax_shape", "relax_vol", "K_POINTS", ""]:
                    continue
                if key[0] == "_":
                    continue
                if "interaction" in key.lower():
                    continue
                incar[key.lower()] = cal_setting[key]

            fix_atom = [False, False, False]
            if cal_type == "relaxation":
                relax_pos = cal_setting["relax_pos"]
                relax_shape = cal_setting["relax_shape"]
                relax_vol = cal_setting["relax_vol"]
                if [relax_pos, relax_shape, relax_vol] == [True, False, False]:
                    self.modify_input(incar, "calculation", "relax")
                elif [relax_pos, relax_shape, relax_vol] == [True, True, True]:
                    self.modify_input(incar, "calculation", "cell-relax")
                elif [relax_pos, relax_shape, relax_vol] == [True, True, False]:
                    self.modify_input(incar, "calculation", "cell-relax")
                    self.modify_input(incar, "fixed_axes", "volume")
                elif [relax_pos, relax_shape, relax_vol] == [False, True, False]:
                    self.modify_input(incar, "calculation", "cell-relax")
                    self.modify_input(incar, "fixed_axes", "volume")
                    fix_atom = [True, True, True]
                elif [relax_pos, relax_shape, relax_vol] == [False, True, True]:
                    self.modify_input(incar, "calculation", "cell-relax")
                    fix_atom = [True, True, True]
                elif [relax_pos, relax_shape, relax_vol] == [False, False, True]:
                    raise RuntimeError(
                        "relax volume but fix shape is not supported for ABACUS"
                    )
                elif [relax_pos, relax_shape, relax_vol] == [False, False, False]:
                    self.modify_input(incar, "calculation", "scf")
                else:
                    raise RuntimeError("not supported calculation setting for ABACUS")

            elif cal_type == "static":
                self.modify_input(incar, "calculation", "scf")

            else:
                raise RuntimeError("not supported calculation type for ABACUS")

            # modify STRU file base on the value of fix_atom
            abacus_utils.stru_fix_atom(os.path.join(output_dir, "STRU"), fix_atom)

        if "basis_type" not in incar:
            logging.info("'basis_type' is not defined, set to be 'pw'!")
            self.modify_input(incar, "basis_type", "pw")
        if "lcao" in incar["basis_type"].lower() and not self.if_define_orb_file:
            mess = (
                "The basis_type is %s, but not define orbital file!!!"
                % incar["basis_type"]
            )
            raise RuntimeError(mess)
        abacus_utils.write_input(os.path.join(output_dir, "../INPUT"), incar)
        cwd = os.getcwd()
        os.chdir(output_dir)
        if not os.path.islink("INPUT"):
            os.symlink("../INPUT", "INPUT")
        elif not "../INPUT" == os.readlink("INPUT"):
            os.remove("INPUT")
            os.symlink("../INPUT", "INPUT")
        os.chdir(cwd)

        if "kspacing" in incar:
            kspacing = float(incar["kspacing"])
            if os.path.isfile(os.path.join(output_dir, "STRU")):
                kpt = abacus_utils.make_kspacing_kpt(
                    os.path.join(output_dir, "STRU"), kspacing
                )
                kpt += [0, 0, 0]
            else:
                kpt = [1, 1, 1, 0, 0, 0]
        elif "K_POINTS" in cal_setting:
            kpt = cal_setting["K_POINTS"]
        else:
            mess = "K point information is not defined\n"
            mess += "You can set key word 'kspacing' (unit in 1/bohr) as a float value in INPUT\n"
            mess += "or set key word 'K_POINTS' as a list in 'cal_setting', e.g. [1,2,3,0,0,0]\n"
            raise RuntimeError(mess)
        abacus_utils.write_kpt(os.path.join(output_dir, "KPT"), kpt)

    def compute(self, output_dir):
        if not os.path.isfile(os.path.join(output_dir, "INPUT")):
            logging.warning("cannot find INPUT in " + output_dir + " skip")
            return None
        ls = LabeledSystem(output_dir, fmt="abacus/relax")
        outcar_dict = ls.as_dict()
        return outcar_dict

    def forward_files(self, property_type="relaxation"):
        return ["INPUT", "STRU", "KPT", "pp_orb"]

    def forward_common_files(self, property_type="relaxation"):
        return []

    def backward_files(self, property_type="relaxation"):
        return []




import os
from dpdispatcher import (
    Machine,
    Resources,
    Submission,
    Task
)
from dflow.python import upload_packages
upload_packages.append(__file__)


def make_submission(
    mdata_machine,
    mdata_resources,
    commands,
    work_path,
    run_tasks,
    group_size,
    forward_common_files,
    forward_files,
    backward_files,
    outlog,
    errlog,
):

    if mdata_machine["local_root"] != "./":
        raise RuntimeError(f"local_root must be './' in dpgen's machine.json.")

    abs_local_root = os.path.abspath("./")

    abs_mdata_machine = mdata_machine.copy()
    abs_mdata_machine["local_root"] = abs_local_root

    machine = Machine.load_from_dict(abs_mdata_machine)
    resources = Resources.load_from_dict(mdata_resources)

    command = "&&".join(commands)

    task_list = []
    for ii in run_tasks:
        task = Task(
            command=command,
            task_work_path=ii,
            forward_files=forward_files,
            backward_files=backward_files,
            outlog=outlog,
            errlog=errlog,
        )
        task_list.append(task)
    submission = Submission(
        work_base=work_path,
        machine=machine,
        resources=resources,
        task_list=task_list,
        forward_common_files=[],
        backward_common_files=[],
    )
    return submission




import os
from pathlib import (
    Path,
)
from typing import (
    List,
    Optional,
    Type,
)
from dflow import (
    InputArtifact,
    InputParameter,
    Inputs,
    OutputArtifact,
    Outputs,
    Step,
    Steps,
    Workflow,
    argo_len,
    argo_range,
    upload_artifact,
)
from dflow.python import (
    OP,
    PythonOPTemplate,
    Slices,
)
from dflow.plugins.dispatcher import DispatcherExecutor


class RelaxationFlow(Steps):

    def __init__(
        self,
        name: str,
        make_op: Type[OP],
        run_op: Type[OP],
        post_op: Type[OP],
        make_image: str,
        run_image: str,
        post_image: str,
        run_command: str,
        calculator: str,
        group_size: Optional[int] = None,
        pool_size: Optional[int] = None,
        executor: Optional[DispatcherExecutor] = None,
        upload_python_packages: Optional[List[os.PathLike]] = None,
    ):
        self._input_parameters = {
            "flow_id": InputParameter(type=str, value=""),
            "parameter": InputParameter(type=dict)
        }
        self._input_artifacts = {
            "input_work_path": InputArtifact(type=Path),
        }
        self._output_parameters = {}
        self._output_artifacts = {
            "output_all": OutputArtifact(type=Path),
            "retrieve_path": OutputArtifact(type=List[Path])
        }

        super().__init__(
            name=name,
            inputs=Inputs(
                parameters=self._input_parameters,
                artifacts=self._input_artifacts
            ),
            outputs=Outputs(
                parameters=self._output_parameters,
                artifacts=self._output_artifacts
            ),
        )

        self._keys = ["make", "run", "post"]
        self.step_keys = {}
        key = "make"
        self.step_keys[key] = '--'.join(
            [str(self.inputs.parameters["flow_id"]), 'relax', key]
        )
        key = "run"
        self.step_keys[key] = '--'.join(
            [str(self.inputs.parameters["flow_id"]), 'relax', key + "-{{item}}"]
        )
        key = "post"
        self.step_keys[key] = '--'.join(
            [str(self.inputs.parameters["flow_id"]), 'relax', key]
        )

        self._build(
            name,
            make_op,
            run_op,
            post_op,
            make_image,
            run_image,
            post_image,
            run_command,
            calculator,
            group_size,
            pool_size,
            executor,
            upload_python_packages
        )

    @property
    def input_parameters(self):
        return self._input_parameters

    @property
    def input_artifacts(self):
        return self._input_artifacts

    @property
    def output_parameters(self):
        return self._output_parameters

    @property
    def output_artifacts(self):
        return self._output_artifacts

    @property
    def keys(self):
        return self._keys

    def _build(
        self,
        name: str,
        make_op: Type[OP],
        run_op: Type[OP],
        post_op: Type[OP],
        make_image: str,
        run_image: str,
        post_image: str,
        run_command: str,
        calculator: str,
        group_size: Optional[int] = None,
        pool_size: Optional[int] = None,
        executor: Optional[DispatcherExecutor] = None,
        upload_python_packages: Optional[List[os.PathLike]] = None,
    ):
        make = Step(
            name="Relaxmake",
            template=PythonOPTemplate(make_op,
                                      image=make_image,
                                      python_packages=upload_python_packages,
                                      command=["python3"]),
            artifacts={"input": self.inputs.artifacts["input_work_path"]},
            parameters={"param": self.inputs.parameters["parameter"]},
            key=self.step_keys["make"]
        )
        self.add(make)

        # Step for Relaxation run
        if calculator in ['vasp', 'abacus']:
            run_fp = PythonOPTemplate(
                run_op,
                slices=Slices(
                    "{{item}}",
                    input_parameter=["task_name"],
                    input_artifact=["task_path"],
                    output_artifact=["backward_dir"],
                    group_size=group_size,
                    pool_size=pool_size
                ),
                python_packages=upload_python_packages,
                image=run_image
            )
        if calculator == 'vasp':
            runcal = Step(
                name="RelaxVASP-Cal",
                template=run_fp,
                parameters={
                    "run_image_config": {"command": run_command},
                    "task_name": make.outputs.parameters["task_names"],
                    "backward_list": ["INCAR", "POSCAR", "OUTCAR", "CONTCAR"],
                    "backward_dir_name": "relax_task"
                },
                artifacts={
                    "task_path": make.outputs.artifacts["task_paths"]
                },
                with_param=argo_range(argo_len(make.outputs.parameters["task_names"])),
                key=self.step_keys["run"] + '-vasp',
                executor=executor,
            )
        elif calculator == 'abacus':
            runcal = Step(
                name="RelaxABACUS-Cal",
                template=run_fp,
                parameters={
                    "run_image_config": {"command": run_command},
                    "task_name": make.outputs.parameters["task_names"],
                    "backward_list": ["OUT.ABACUS", "log"],
                    "log_name": "log",
                    "backward_dir_name": "relax_task"
                },
                artifacts={
                    "task_path": make.outputs.artifacts["task_paths"],
                    "optional_artifact": upload_artifact({"pp_orb": "./"})
                },
                with_param=argo_range(argo_len(make.outputs.parameters["task_names"])),
                key=self.step_keys["run"] + '-abacus',
                executor=executor,
            )
        elif calculator == 'lammps':
            run_lmp = PythonOPTemplate(
                run_op,
                slices=Slices(
                    "{{item}}",
                    input_artifact=["input_lammps"],
                    output_artifact=["backward_dir"],
                    group_size=group_size,
                    pool_size=pool_size
                ),
                image=run_image,
                python_packages=upload_python_packages,
                command=["python3"]
            )
            runcal = Step(
                name="RelaxLAMMPS-Cal",
                template=run_lmp,
                artifacts={"input_lammps": make.outputs.artifacts["task_paths"]},
                parameters={"run_command": run_command},
                with_param=argo_range(make.outputs.parameters["njobs"]),
                key=self.step_keys["run"] + '-lammps',
                executor=executor,
            )
        else:
            raise RuntimeError(f'Incorrect calculator type to initiate step: {calculator}')
        self.add(runcal)

        post = Step(
            name="Relaxpost",
            template=PythonOPTemplate(post_op,
                                      image=post_image,
                                      command=["python3"]),
            artifacts={"input_post": runcal.outputs.artifacts["backward_dir"],
                       "input_all": make.outputs.artifacts["output"]},
            parameters={"param": self.inputs.parameters["parameter"]},
            key=self.step_keys["post"]
        )
        self.add(post)

        self.outputs.artifacts["output_all"]._from \
            = post.outputs.artifacts["output_all"]
        self.outputs.artifacts["retrieve_path"]._from \
            = post.outputs.artifacts["retrieve_path"]





import glob
import logging
import os
import re
from shutil import copyfile

from monty.serialization import dumpfn, loadfn
from pymatgen.analysis.elasticity.elastic import ElasticTensor
from pymatgen.analysis.elasticity.strain import DeformedStructureSet, Strain
from pymatgen.analysis.elasticity.stress import Stress
from pymatgen.core.structure import Structure
from pymatgen.io.vasp import Incar, Kpoints

from apex.core.calculator.lib import abacus_utils
from apex.core.calculator.lib import vasp_utils
from apex.core.calculator.lib import abacus_scf
from apex.core.property.Property import Property
from apex.core.refine import make_refine
from apex.core.calculator.lib.vasp_utils import incar_upper
from dflow.python import upload_packages
upload_packages.append(__file__)


class Elastic(Property):
    def __init__(self, parameter, inter_param=None):
        if not ("init_from_suffix" in parameter and "output_suffix" in parameter):
            default_norm_def = 1e-2
            default_shear_def = 1e-2
            parameter["norm_deform"] = parameter.get("norm_deform", default_norm_def)
            self.norm_deform = parameter["norm_deform"]
            parameter["shear_deform"] = parameter.get("shear_deform", default_shear_def)
            self.shear_deform = parameter["shear_deform"]
        parameter["cal_type"] = parameter.get("cal_type", "relaxation")
        self.cal_type = parameter["cal_type"]
        default_cal_setting = {
            "relax_pos": True,
            "relax_shape": False,
            "relax_vol": False,
        }
        if "cal_setting" not in parameter:
            parameter["cal_setting"] = default_cal_setting
        else:
            if "relax_pos" not in parameter["cal_setting"]:
                parameter["cal_setting"]["relax_pos"] = default_cal_setting["relax_pos"]
            if "relax_shape" not in parameter["cal_setting"]:
                parameter["cal_setting"]["relax_shape"] = default_cal_setting[
                    "relax_shape"
                ]
            if "relax_vol" not in parameter["cal_setting"]:
                parameter["cal_setting"]["relax_vol"] = default_cal_setting["relax_vol"]
        self.cal_setting = parameter["cal_setting"]
        # parameter['reproduce'] = False
        # self.reprod = parameter['reproduce']
        self.parameter = parameter
        self.inter_param = inter_param if inter_param != None else {"type": "vasp"}

    def make_confs(self, path_to_work, path_to_equi, refine=False):
        path_to_work = os.path.abspath(path_to_work)
        if os.path.exists(path_to_work):
            logging.warning("%s already exists" % path_to_work)
            #dlog.warning("%s already exists" % path_to_work)
        else:
            os.makedirs(path_to_work)
        path_to_equi = os.path.abspath(path_to_equi)

        if "start_confs_path" in self.parameter and os.path.exists(
            self.parameter["start_confs_path"]
        ):
            init_path_list = glob.glob(
                os.path.join(self.parameter["start_confs_path"], "*")
            )
            struct_init_name_list = []
            for ii in init_path_list:
                struct_init_name_list.append(ii.split("/")[-1])
            struct_output_name = path_to_work.split("/")[-2]
            assert struct_output_name in struct_init_name_list
            path_to_equi = os.path.abspath(
                os.path.join(
                    self.parameter["start_confs_path"],
                    struct_output_name,
                    "relaxation",
                    "relax_task",
                )
            )

        task_list = []
        cwd = os.getcwd()

        if self.inter_param["type"] == "abacus":
            CONTCAR = abacus_utils.final_stru(path_to_equi)
            POSCAR = "STRU"
        else:
            CONTCAR = "CONTCAR"
            POSCAR = "POSCAR"

        equi_contcar = os.path.join(path_to_equi, CONTCAR)

        os.chdir(path_to_work)
        if os.path.isfile(POSCAR):
            os.remove(POSCAR)
        if os.path.islink(POSCAR):
            os.remove(POSCAR)
        os.symlink(os.path.relpath(equi_contcar), POSCAR)
        #           task_poscar = os.path.join(output, 'POSCAR')

        # stress, deal with unsupported stress in dpdata
        # with open(os.path.join(path_to_equi, 'result.json')) as fin:
        #    equi_result = json.load(fin)
        # equi_stress = np.array(equi_result['stress']['data'])[-1]
        equi_result = loadfn(os.path.join(path_to_equi, "result.json"))
        equi_stress = equi_result["stress"][-1]
        dumpfn(equi_stress, "equi.stress.json", indent=4)
        os.chdir(cwd)

        if refine:
            print("elastic refine starts")
            task_list = make_refine(
                self.parameter["init_from_suffix"],
                self.parameter["output_suffix"],
                path_to_work,
            )

            # record strain
            # df = Strain.from_deformation(dfm_ss.deformations[idid])
            # dumpfn(df.as_dict(), 'strain.json', indent=4)
            init_from_path = re.sub(
                self.parameter["output_suffix"][::-1],
                self.parameter["init_from_suffix"][::-1],
                path_to_work[::-1],
                count=1,
            )[::-1]
            task_list_basename = list(map(os.path.basename, task_list))

            for ii in task_list_basename:
                init_from_task = os.path.join(init_from_path, ii)
                output_task = os.path.join(path_to_work, ii)
                os.chdir(output_task)
                if os.path.isfile("strain.json"):
                    os.remove("strain.json")
                copyfile(os.path.join(init_from_task, "strain.json"), "strain.json")
                # os.symlink(os.path.relpath(
                #    os.path.join((re.sub(self.parameter['output_suffix'], self.parameter['init_from_suffix'], ii)),
                #                 'strain.json')),
                #           'strain.json')
            os.chdir(cwd)
        else:
            norm_def = self.norm_deform
            shear_def = self.shear_deform
            norm_strains = [-norm_def, -0.5 * norm_def, 0.5 * norm_def, norm_def]
            shear_strains = [-shear_def, -0.5 * shear_def, 0.5 * shear_def, shear_def]

            if not os.path.exists(equi_contcar):
                raise RuntimeError("please do relaxation first")

            if self.inter_param["type"] == "abacus":
                ss = abacus_utils.stru2Structure(equi_contcar)
            else:
                ss = Structure.from_file(equi_contcar)
            dfm_ss = DeformedStructureSet(
                ss,
                symmetry=False,
                norm_strains=norm_strains,
                shear_strains=shear_strains,
            )
            n_dfm = len(dfm_ss)

            print("gen with norm " + str(norm_strains))
            print("gen with shear " + str(shear_strains))
            for ii in range(n_dfm):
                output_task = os.path.join(path_to_work, "task.%06d" % ii)
                os.makedirs(output_task, exist_ok=True)
                os.chdir(output_task)
                for jj in [
                    "INCAR",
                    "POTCAR",
                    "POSCAR",
                    "conf.lmp",
                    "in.lammps",
                    "STRU",
                ]:
                    if os.path.exists(jj):
                        os.remove(jj)
                task_list.append(output_task)
                dfm_ss.deformed_structures[ii].to("POSCAR", "POSCAR")
                if self.inter_param["type"] == "abacus":
                    abacus_utils.poscar2stru("POSCAR", self.inter_param, "STRU")
                    os.remove("POSCAR")
                # record strain
                df = Strain.from_deformation(dfm_ss.deformations[ii])
                dumpfn(df.as_dict(), "strain.json", indent=4)
            os.chdir(cwd)
        return task_list

    def post_process(self, task_list):
        if self.inter_param["type"] == "abacus":
            POSCAR = "STRU"
            INCAR = "INPUT"
            KPOINTS = "KPT"
        else:
            POSCAR = "POSCAR"
            INCAR = "INCAR"
            KPOINTS = "KPOINTS"

        cwd = os.getcwd()
        poscar_start = os.path.abspath(os.path.join(task_list[0], "..", POSCAR))
        os.chdir(os.path.join(task_list[0], ".."))
        if os.path.isfile(os.path.join(task_list[0], INCAR)):
            if self.inter_param["type"] == "abacus":
                input_aba = abacus_scf.get_abacus_input_parameters("INPUT")
                if "kspacing" in input_aba:
                    kspacing = float(input_aba["kspacing"])
                    kpt = abacus_utils.make_kspacing_kpt(poscar_start, kspacing)
                    kpt += [0, 0, 0]
                    abacus_utils.write_kpt("KPT", kpt)
                    del input_aba["kspacing"]
                    os.remove("INPUT")
                    abacus_utils.write_input("INPUT", input_aba)
                else:
                    os.rename(os.path.join(task_list[0], "KPT"), "./KPT")
            else:
                incar = incar_upper(
                    Incar.from_file(os.path.join(task_list[0], "INCAR"))
                )
                kspacing = incar.get("KSPACING")
                kgamma = incar.get("KGAMMA", False)
                ret = vasp_utils.make_kspacing_kpoints(poscar_start, kspacing, kgamma)
                kp = Kpoints.from_string(ret)
                if os.path.isfile("KPOINTS"):
                    os.remove("KPOINTS")
                kp.write_file("KPOINTS")

            os.chdir(cwd)
            kpoints_universal = os.path.abspath(
                os.path.join(task_list[0], "..", KPOINTS)
            )
            for ii in task_list:
                if os.path.isfile(os.path.join(ii, KPOINTS)):
                    os.remove(os.path.join(ii, KPOINTS))
                if os.path.islink(os.path.join(ii, KPOINTS)):
                    os.remove(os.path.join(ii, KPOINTS))
                os.chdir(ii)
                os.symlink(os.path.relpath(kpoints_universal), KPOINTS)

        os.chdir(cwd)

    def task_type(self):
        return self.parameter["type"]

    def task_param(self):
        return self.parameter

    def _compute_lower(self, output_file, all_tasks, all_res):
        output_file = os.path.abspath(output_file)
        res_data = {}
        ptr_data = os.path.dirname(output_file) + "\n"
        equi_stress = Stress(
            loadfn(os.path.join(os.path.dirname(output_file), "equi.stress.json"))
        )
        equi_stress *= -1000
        lst_strain = []
        lst_stress = []
        for ii in all_tasks:
            strain = loadfn(os.path.join(ii, "strain.json"))
            # stress, deal with unsupported stress in dpdata
            # with open(os.path.join(ii, 'result_task.json')) as fin:
            #    task_result = json.load(fin)
            # stress = np.array(task_result['stress']['data'])[-1]
            stress = loadfn(os.path.join(ii, "result_task.json"))["stress"][-1]
            lst_strain.append(strain)
            lst_stress.append(Stress(stress * -1000))

        et = ElasticTensor.from_independent_strains(
            lst_strain, lst_stress, eq_stress=equi_stress, vasp=False
        )
        res_data["elastic_tensor"] = []
        for ii in range(6):
            for jj in range(6):
                res_data["elastic_tensor"].append(et.voigt[ii][jj] / 1e4)
                ptr_data += "%7.2f " % (et.voigt[ii][jj] / 1e4)
            ptr_data += "\n"

        BV = et.k_voigt / 1e4
        GV = et.g_voigt / 1e4
        EV = 9 * BV * GV / (3 * BV + GV)
        uV = 0.5 * (3 * BV - 2 * GV) / (3 * BV + GV)

        res_data["BV"] = BV
        res_data["GV"] = GV
        res_data["EV"] = EV
        res_data["uV"] = uV
        ptr_data += "# Bulk   Modulus BV = %.2f GPa\n" % BV
        ptr_data += "# Shear  Modulus GV = %.2f GPa\n" % GV
        ptr_data += "# Youngs Modulus EV = %.2f GPa\n" % EV
        ptr_data += "# Poission Ratio uV = %.2f\n " % uV

        dumpfn(res_data, output_file, indent=4)

        return res_data, ptr_data



import glob
import os

import numpy as np
from monty.serialization import loadfn

from apex.core.calculator.lib import abacus_utils
from dflow.python import upload_packages
upload_packages.append(__file__)


def make_repro(
    inter_param, init_data_path, init_from_suffix, path_to_work, reprod_last_frame=True
):
    path_to_work = os.path.abspath(path_to_work)
    property_type = path_to_work.split("/")[-1].split("_")[0]
    init_data_path = os.path.join(
        init_data_path, "*", property_type + "_" + init_from_suffix
    )
    init_data_path_list = glob.glob(init_data_path)
    init_data_path_list.sort()
    cwd = os.getcwd()
    struct_init_name_list = []
    for ii in init_data_path_list:
        struct_init_name_list.append(ii.split("/")[-2])
    struct_output_name = path_to_work.split("/")[-2]

    assert struct_output_name in struct_init_name_list

    for idx, ii in enumerate(struct_init_name_list):
        if ii == struct_output_name:
            label = idx

    init_data_path_todo = init_data_path_list[label]

    init_data_task_todo = glob.glob(
        os.path.join(init_data_path_todo, "task.[0-9]*[0-9]")
    )
    assert (
        len(init_data_task_todo) > 0
    ), "There is no task in previous calculations path"
    init_data_task_todo.sort()

    task_list = []
    task_num = 0

    if property_type == "interstitial":
        if os.path.exists(os.path.join(path_to_work, "element.out")):
            os.remove(os.path.join(path_to_work, "element.out"))
        fout_element = open(os.path.join(path_to_work, "element.out"), "a+")
        fin_element = open(os.path.join(init_data_path_todo, "element.out"), "r")

    for ii in init_data_task_todo:
        # get frame number
        task_result = loadfn(os.path.join(ii, "result_task.json"))
        if reprod_last_frame:
            nframe = 1
        else:
            nframe = len(task_result["energies"])
        if property_type == "interstitial":
            insert_element = fin_element.readline().split()[0]
        for jj in range(nframe):
            if property_type == "interstitial":
                print(insert_element, file=fout_element)
            output_task = os.path.join(path_to_work, "task.%06d" % task_num)
            task_num += 1
            task_list.append(output_task)
            os.makedirs(output_task, exist_ok=True)
            os.chdir(output_task)
            # clear dir
            for kk in [
                "INCAR",
                "POTCAR",
                "POSCAR.orig",
                "POSCAR",
                "conf.lmp",
                "in.lammps",
                "STRU",
            ]:
                if os.path.exists(kk):
                    os.remove(kk)
            # make conf
            if reprod_last_frame:
                task_result.to("vasp/poscar", "POSCAR", frame_idx=-1)
            else:
                task_result.to("vasp/poscar", "POSCAR", frame_idx=jj)
            if inter_param["type"] == "abacus":
                abacus_utils.poscar2stru("POSCAR", inter_param, "STRU")
                os.remove("POSCAR")

    os.chdir(cwd)

    if property_type == "interstitial":
        fout_element.close()
        fin_element.close()

    return task_list


def post_repro(
    init_data_path, init_from_suffix, all_tasks, ptr_data, reprod_last_frame=True
):
    ptr_data += "Reproduce: Initial_path Init_E(eV/atom)  Reprod_E(eV/atom)  Difference(eV/atom)\n"
    struct_output_name = all_tasks[0].split("/")[-3]
    property_type = all_tasks[0].split("/")[-2].split("_")[0]
    init_data_path = os.path.join(
        init_data_path, "*", property_type + "_" + init_from_suffix
    )
    init_data_path_list = glob.glob(init_data_path)
    init_data_path_list.sort()
    # cwd = os.getcwd()
    struct_init_name_list = []
    for ii in init_data_path_list:
        struct_init_name_list.append(ii.split("/")[-2])

    assert struct_output_name in struct_init_name_list

    for idx, ii in enumerate(struct_init_name_list):
        if ii == struct_output_name:
            label = idx

    init_data_path_todo = init_data_path_list[label]

    init_data_task_todo = glob.glob(
        os.path.join(init_data_path_todo, "task.[0-9]*[0-9]")
    )
    assert (
        len(init_data_task_todo) > 0
    ), "There is no task in previous calculations path"
    init_data_task_todo.sort()

    idid = 0
    init_ener_tot = []
    output_ener_tot = []
    res_data = {}

    for ii in init_data_task_todo:
        init_task_result = loadfn(os.path.join(ii, "result_task.json"))
        if reprod_last_frame:
            nframe = 1
        else:
            nframe = len(init_task_result["energies"])
        # idid += nframe
        natoms = np.sum(init_task_result["atom_numbs"])
        if reprod_last_frame:
            init_ener = init_task_result["energies"][-1:]
        else:
            init_ener = init_task_result["energies"]
        init_ener_tot.extend(list(init_ener))
        output_ener = []
        for jj in range(idid, idid + nframe):
            output_task_result = loadfn(os.path.join(all_tasks[jj], "result_task.json"))
            output_epa = output_task_result["energies"] / natoms
            output_ener.append(output_epa)
            output_ener_tot.extend(output_task_result["energies"])

            init_epa = init_ener[jj - idid] / natoms
            ptr_data += "%s %7.3f  %7.3f  %7.3f\n" % (
                ii,
                init_epa,
                output_epa,
                output_epa - init_epa,
            )
        idid += nframe
        output_ener = np.array(output_ener)
        output_ener = np.reshape(output_ener, [-1, 1])
        init_ener = np.reshape(init_ener, [-1, 1]) / natoms
        if reprod_last_frame:
            error_start = 0
        else:
            error_start = 1
            output_ener -= output_ener[-1] - init_ener[-1]
        diff = output_ener - init_ener
        diff = diff[error_start:]
        error = np.linalg.norm(diff) / np.sqrt(np.size(output_ener) - error_start)
        res_data[ii] = {"nframes": len(init_ener), "error": error}

    if not len(init_ener_tot) == len(output_ener_tot):
        raise RuntimeError("reproduce tasks not equal to init")
    #    for ii in range(len(lmp_ener_tot)):
    #        ptr_data += '%7.3f  %7.3f  %7.3f\n' % (vasp_ener_tot[ii], lmp_ener_tot[ii],
    #                                               lmp_ener_tot[ii] - vasp_ener_tot[ii])
    return res_data, ptr_data



import glob
import os
import shutil
import sys
import unittest

from pymatgen.core import Structure
from pymatgen.io.vasp import Incar

from apex.core.property.Elastic import Elastic

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestElastic(unittest.TestCase):
    def setUp(self):
        _jdata = {
            "structures": ["confs/std-fcc"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR.rlx",
                "potcar_prefix": ".",
                "potcars": {"Al": "vasp_input/POT_Al"},
            },
            "properties": [
                {
                    "skip": False,
                    "type": "elastic",
                    "norm_deform": 2e-2,
                    "shear_deform": 5e-2,
                }
            ],
        }

        self.equi_path = "confs/std-fcc/relaxation/task_relax"
        self.source_path = "equi/vasp"
        self.target_path = "confs/std-fcc/elastic_00"
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata["properties"]

        self.elastic = Elastic(_jdata["properties"][0])

    def tearDown(self):
        if os.path.exists(os.path.join(self.equi_path, "..")):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

    def test_task_type(self):
        self.assertEqual("elastic", self.elastic.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.elastic.task_param())

    def test_make_confs(self):

        shutil.copy(
            os.path.join(self.source_path, "Al-fcc.json"),
            os.path.join(self.equi_path, "result.json"),
        )
        if not os.path.exists(os.path.join(self.equi_path, "CONTCAR")):
            with self.assertRaises(RuntimeError):
                self.elastic.make_confs(self.target_path, self.equi_path)
        shutil.copy(
            os.path.join(self.source_path, "CONTCAR_Al_fcc"),
            os.path.join(self.equi_path, "CONTCAR"),
        )
        task_list = self.elastic.make_confs(self.target_path, self.equi_path)
        dfm_dirs = glob.glob(os.path.join(self.target_path, "task.*"))

        incar0 = Incar.from_file(os.path.join("vasp_input", "INCAR.rlx"))
        incar0["ISIF"] = 4

        self.assertEqual(
            os.path.realpath(os.path.join(self.equi_path, "CONTCAR")),
            os.path.realpath(os.path.join(self.target_path, "POSCAR")),
        )
        ref_st = Structure.from_file(os.path.join(self.target_path, "POSCAR"))
        dfm_dirs.sort()
        for ii in dfm_dirs:
            st_file = os.path.join(ii, "POSCAR")
            self.assertTrue(os.path.isfile(st_file))
            strain_json_file = os.path.join(ii, "strain.json")
            self.assertTrue(os.path.isfile(strain_json_file))



import glob
import os
import shutil
import sys
import unittest

import numpy as np
from pymatgen.analysis.defects.core import Interstitial as pmg_Interstitial
from pymatgen.core import Structure
from pymatgen.io.vasp import Incar

from apex.core.property.Interstitial import Interstitial

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestInterstitial(unittest.TestCase):
    def setUp(self):
        _jdata = {
            "structures": ["confs/std-bcc"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR.rlx",
                "potcar_prefix": "vasp_input",
                "potcars": {"V": "POTCAR"},
            },
            "properties": [
                {
                    "type": "interstitial",
                    "supercell": [1, 1, 1],
                    "insert_ele": ["V"],
                    "bcc_self": True,
                }
            ],
        }

        self.equi_path = "confs/std-bcc/relaxation/relax_task"
        self.source_path = "equi/vasp"
        self.target_path = "confs/std-bcc/interstitial_00"
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)
        if not os.path.exists(self.target_path):
            os.makedirs(self.target_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata["properties"]

        self.interstitial = Interstitial(_jdata["properties"][0])

    def tearDown(self):
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

    def test_task_type(self):
        self.assertEqual("interstitial", self.interstitial.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.interstitial.task_param())

    def test_make_confs_bcc(self):
        if not os.path.exists(os.path.join(self.equi_path, "CONTCAR")):
            with self.assertRaises(RuntimeError):
                self.interstitial.make_confs(self.target_path, self.equi_path)
        shutil.copy(
            os.path.join(self.source_path, "CONTCAR_V_bcc"),
            os.path.join(self.equi_path, "CONTCAR"),
        )
        task_list = self.interstitial.make_confs(self.target_path, self.equi_path)
        dfm_dirs = glob.glob(os.path.join(self.target_path, "task.*"))
        self.assertEqual(len(dfm_dirs), 7)

        incar0 = Incar.from_file(os.path.join("vasp_input", "INCAR.rlx"))
        incar0["ISIF"] = 3

        self.assertEqual(
            os.path.realpath(os.path.join(self.equi_path, "CONTCAR")),
            os.path.realpath(os.path.join(self.target_path, "POSCAR")),
        )
        ref_st = Structure.from_file(os.path.join(self.target_path, "POSCAR"))
        dfm_dirs.sort()
        for ii in dfm_dirs[:4]:
            st_file = os.path.join(ii, "POSCAR")
            self.assertTrue(os.path.isfile(st_file))
            st0 = Structure.from_file(st_file)
            inter_site = st0[0]
            inter = pmg_Interstitial(ref_st, inter_site)
            st1 = inter.get_supercell_structure(
                sc_mat=np.eye(3) * self.prop_param[0]["supercell"]
            )
            st0_coords = [list(st0.sites[ii].coords) for ii in range(3)]
            st1_coords = [list(st1.sites[ii].coords) for ii in range(3)]
            self.assertEqual(st0_coords, st1_coords)

        for ii in dfm_dirs[4:]:
            st_file = os.path.join(ii, "POSCAR")
            self.assertTrue(os.path.isfile(st_file))
            st0 = Structure.from_file(st_file)
            inter_site1 = st0.pop(0)
            inter_site2 = st0.pop(-1)
            center = (inter_site1.coords + inter_site2.coords) / 2
            self.assertTrue((center[0] - center[1]) < 1e-4)
            self.assertTrue((center[1] - center[2]) < 1e-4)



import glob
import json
import os
from abc import ABC, abstractmethod

from monty.serialization import dumpfn

from apex.core.calculator.calculator import make_calculator
from dflow.python import upload_packages
upload_packages.append(__file__)


class Property(ABC):
    @abstractmethod
    def __init__(self, parameter):
        """
        Constructor

        Parameters
        ----------
        parameter : dict
            A dict that defines the apex.
        """
        pass

    @abstractmethod
    def make_confs(self, path_to_work, path_to_equi, refine=False):
        """
        Make configurations needed to compute the apex.
        The tasks directory will be named as path_to_work/task.xxxxxx
        IMPORTANT: handel the case when the directory exists.

        Parameters
        ----------
        path_to_work : str
            The path where the tasks for the apex are located
        path_to_equi : str
            -refine == False: The path to the directory that equilibrated the configuration.
            -refine == True: The path to the directory that has apex confs.
        refine : str
            To refine existing apex confs or generate apex confs from a equilibrated conf

        Returns
        -------
        task_list: list of str
            The list of task directories.
        """
        pass

    @abstractmethod
    def post_process(self, task_list):
        """
        post_process the KPOINTS file in elastic.
        """
        pass

    @property
    @abstractmethod
    def task_type(self):
        """
        Return the type of each computational task, for example, 'relaxation', 'static'....
        """
        pass

    @property
    @abstractmethod
    def task_param(self):
        """
        Return the parameter of each computational task, for example, {'ediffg': 1e-4}
        """
        pass

    def compute(self, output_file, print_file, path_to_work):
        """
        Postprocess the finished tasks to compute the apex.
        Output the result to a json database

        Parameters
        ----------
        output_file:
            The file to output the apex in json format
        print_file:
            The file to output the apex in txt format
        path_to_work:
            The working directory where the computational tasks locate.
        """
        path_to_work = os.path.abspath(path_to_work)
        task_dirs = glob.glob(os.path.join(path_to_work, "task.[0-9]*[0-9]"))
        task_dirs.sort()
        all_res = []
        for ii in task_dirs:
            with open(os.path.join(ii, "inter.json")) as fp:
                idata = json.load(fp)
            poscar = os.path.join(ii, "POSCAR")
            task = make_calculator(idata, poscar)
            res = task.compute(ii)
            dumpfn(res, os.path.join(ii, "result_task.json"), indent=4)
            # all_res.append(res)
            all_res.append(os.path.join(ii, "result_task.json"))

        # cwd = os.getcwd()
        # os.chdir(path_to_work)
        res, ptr = self._compute_lower(output_file, task_dirs, all_res)
        #        with open(output_file, 'w') as fp:
        #            json.dump(fp, res, indent=4)
        with open(print_file, "w") as fp:
            fp.write(ptr)
        # os.chdir(cwd)

    @abstractmethod
    def _compute_lower(self, output_file, all_tasks, all_res):
        """
        Compute the apex.

        Parameters
        ----------
        output_file:
            The file to output the apex
        all_tasks : list of str
            The list of directories to the tasks
        all_res : list of str
            The list of results
        Returns:
        -------
        res_data : dist
            The dict storing the result of the apex
        ptr_data : str
            The result printed in string format
        """
        pass



import unittest
import sys
import os
import glob
import shutil
from pathlib import Path
from dflow.python import (
    OP,
    OPIO,
    OPIOSign,
    Artifact,
    TransientError,
)
from monty.serialization import loadfn

from apex.op.relaxation_ops import RelaxMake
from apex.op.property_ops import PropsMake
from context import write_poscar

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
__package__ = "tests"


class TestMakeRelaxOPs(unittest.TestCase):
    def setUp(self) -> None:
        cwd = os.getcwd()
        self.path = cwd
        self.vasp_dir = cwd/Path('vasp_input')
        self.abacus_dir = cwd/Path('abacus_input')
        self.lammps_dir = cwd/Path('lammps_input')

        os.chdir(self.vasp_dir)
        shutil.copytree(os.path.join(cwd, 'confs/confs_example0/'), './confs/', dirs_exist_ok=True)
        self.vasp_confs = self.vasp_dir/'confs'
        os.chdir(cwd)

        os.chdir(self.abacus_dir)
        shutil.copytree(os.path.join(cwd, 'confs/confs_abacus_example0/'), './confs/', dirs_exist_ok=True)
        self.abacus_confs = self.abacus_dir/'confs'
        os.chdir(cwd)

        os.chdir(self.lammps_dir)
        shutil.copytree(os.path.join(cwd, 'confs/confs_example0/'), './confs/', dirs_exist_ok=True)
        self.lammps_confs = self.lammps_dir/'confs'
        os.chdir(cwd)

    def tearDown(self) -> None:
        shutil.rmtree(self.vasp_confs)
        shutil.rmtree(self.abacus_confs)
        shutil.rmtree(self.lammps_confs)

    def test_vasp_make_equi(self):
        os.chdir(self.vasp_dir)
        op = RelaxMake()
        out = op.execute(
            OPIO({
            'input': self.vasp_dir,
            'param': loadfn('param_joint.json')
        }))
        os.chdir('..')
        self.assertTrue(os.path.exists(self.vasp_dir/'confs'))
        self.assertTrue(os.path.exists(self.vasp_dir/'confs/std-bcc/relaxation/relax_task'))
        self.assertEqual(out['task_paths'], [self.vasp_dir/'confs/std-bcc/relaxation/relax_task'])

    def test_abacus_make_equi(self):
        os.chdir(self.abacus_dir)
        op = RelaxMake()
        out = op.execute(
            OPIO({
            'input': self.abacus_dir,
            'param': loadfn('param_joint.json')
        }))
        os.chdir('..')
        self.assertTrue(os.path.exists(self.abacus_dir/'confs'))
        self.assertTrue(os.path.exists(self.abacus_dir/'confs/fcc-Al/relaxation/relax_task'))
        self.assertEqual(out['task_paths'], [self.abacus_dir/'confs/fcc-Al/relaxation/relax_task'])

    def test_lammps_make_equi(self):
        os.chdir(self.lammps_dir)
        op = RelaxMake()
        out = op.execute(
            OPIO({
            'input': self.lammps_dir,
            'param': loadfn('param_joint.json')
        }))
        os.chdir('..')
        self.assertTrue(os.path.exists(self.lammps_dir/'confs'))
        self.assertTrue(os.path.exists(self.lammps_dir/'confs/std-bcc/relaxation/relax_task'))
        self.assertEqual(out['task_paths'], [self.lammps_dir/'confs/std-bcc/relaxation/relax_task'])


class TestMakePropsOPs(unittest.TestCase):
    def setUp(self) -> None:
        cwd = os.getcwd()
        self.path = cwd
        self.vasp_dir = cwd/Path('vasp_input')
        self.abacus_dir = cwd/Path('abacus_input')
        self.lammps_dir = cwd/Path('lammps_input')

        os.chdir(self.vasp_dir)
        shutil.copytree(os.path.join(cwd, 'confs/confs_example2/'), './confs/', dirs_exist_ok=True)
        self.vasp_confs = self.vasp_dir/'confs'
        os.chdir(cwd)

        os.chdir(self.abacus_dir)
        shutil.copytree(os.path.join(cwd, 'confs/confs_abacus_example2/'), './confs/', dirs_exist_ok=True)
        self.abacus_confs = self.abacus_dir/'confs'
        os.chdir(cwd)

        os.chdir(self.lammps_dir)
        shutil.copytree(os.path.join(cwd, 'confs/confs_example2/'), './confs/', dirs_exist_ok=True)
        self.lammps_confs = self.lammps_dir/'confs'
        os.chdir(cwd)

    def tearDown(self) -> None:
        shutil.rmtree(self.vasp_confs)
        shutil.rmtree(self.abacus_confs)
        shutil.rmtree(self.lammps_confs)

    def test_vasp_make_props(self):
        os.chdir(self.vasp_dir)
        op = PropsMake()
        out = op.execute(
            OPIO({
                'input_work_path': self.vasp_dir,
                'path_to_prop': 'confs/std-bcc/eos_00',
                'prop_param': loadfn('param_joint.json')['properties'][0],
                'inter_param': loadfn('param_joint.json')['interaction'],
                'do_refine': False
            }))
        os.chdir('..')
        self.assertTrue(os.path.exists(self.vasp_dir/'confs'))
        self.assertTrue(os.path.exists(self.vasp_dir/'confs/std-bcc/eos_00'))
        self.assertEqual(len(out['task_paths']), 2)

    def test_abacus_make_props(self):
        os.chdir(self.abacus_dir)
        op = PropsMake()
        out = op.execute(
            OPIO({
                'input_work_path': self.abacus_dir,
                'path_to_prop': 'confs/fcc-Al/eos_00',
                'prop_param': loadfn('param_joint.json')['properties'][0],
                'inter_param': loadfn('param_joint.json')['interaction'],
                'do_refine': False
            }))
        os.chdir('..')
        self.assertTrue(os.path.exists(self.abacus_dir/'confs'))
        self.assertTrue(os.path.exists(self.abacus_dir/'confs/fcc-Al/eos_00'))
        self.assertEqual(len(out['task_paths']), 2)

    def test_lammps_make_props(self):
        os.chdir('lammps_input')
        op = PropsMake()
        out = op.execute(
            OPIO({
                'input_work_path': self.lammps_dir,
                'path_to_prop': 'confs/std-bcc/eos_00',
                'prop_param': loadfn('param_joint.json')['properties'][0],
                'inter_param': loadfn('param_joint.json')['interaction'],
                'do_refine': False
            }))
        os.chdir('..')
        self.assertTrue(os.path.exists(self.lammps_dir/'confs'))
        self.assertTrue(os.path.exists(self.lammps_dir/'confs/std-bcc/eos_00'))
        self.assertEqual(len(out['task_paths']), 2)



